@@ -100,7 +100,8 @@
 ;;; variable.
 (eval-when (:compile-toplevel :load-toplevel)
   (setf *c-functions-in-runtime*
-        '`(#+netbsd ,@("stat" "lstat" "fstat" "readdir" "opendir"))))
+        #+netbsd '("stat" "lstat" "fstat" "readdir" "opendir")
+        #-netbsd '()))
 
 
 ;;; filesystem access
@@ -144,21 +145,18 @@
 (define-call #-netbsd "opendir" #+netbsd "_opendir"
     (* t) null-alien (pathname filename))
 (define-call* "write" int minusp
-    (fd file-descriptor) (buf (* t)) (count int))
+  (fd file-descriptor) (buf (* t)) (count int))
+
+;;; FIXME: to detect errors in readdir errno needs to be set to 0 and
+;;; then checked, like it's done in sb-unix:readdir.
 #+inode64
 (define-call ("readdir" :c-name "readdir$INODE64" :options :largefile)
   (* dirent)
-  ;; readdir() has the worst error convention in the world.  It's just
-  ;; too painful to support.  (return is NULL _and_ errno "unchanged"
-  ;; is not an error, it's EOF).
   not
   (dir (* t)))
 #-inode64
 (define-call (#-netbsd "readdir" #+netbsd "_readdir" :options :largefile)
   (* dirent)
-  ;; readdir() has the worst error convention in the world.  It's just
-  ;; too painful to support.  (return is NULL _and_ errno "unchanged"
-  ;; is not an error, it's EOF).
   not
   (dir (* t)))
 (define-call "closedir" int minusp (dir (* t)))
@@ -225,6 +223,7 @@
                         (unsupported-warning ',lisp-name ,c-name)
                         form)
                       (export ',lisp-name)))))
+    ;; FIXME: The man page for it says "Never use mktemp()"
     (def-mk*temp mktemp "mktemp" (* char) null-alien nil nil)
     ;; FIXME: Windows does have _mktemp, which has a slightly different
     ;; interface
@@ -281,8 +280,6 @@
 
   ;; uid, gid
   (define-call "geteuid" uid-t never-fails) ; "always successful", it says
-  #-sunos
-  (define-call "getresuid" uid-t never-fails)
   (define-call "getuid" uid-t never-fails)
   (define-call "seteuid" int minusp (uid uid-t))
   #-sunos
@@ -294,7 +291,31 @@
   (define-call "getegid" gid-t never-fails)
   (define-call "getgid" gid-t never-fails)
   #-sunos
-  (define-call "getresgid" gid-t never-fails)
+  (progn
+    (export '(getresgid getresuid) :sb-posix)
+    (declaim (inline getresgid getresuid))
+    (defun getresgid ()
+      (with-alien ((rgid gid-t)
+                   (egid gid-t)
+                   (sgid gid-t))
+        (let ((r
+                (alien-funcall (extern-alien "getresgid"
+                                             (function int (* gid-t) (* gid-t) (* gid-t)))
+                               (addr rgid) (addr egid) (addr sgid))))
+          (if (minusp r)
+              (syscall-error 'getresgid)
+              (values rgid egid sgid)))))
+    (defun getresuid ()
+      (with-alien ((ruid uid-t)
+                   (euid uid-t)
+                   (suid uid-t))
+        (let ((r
+                (alien-funcall (extern-alien "getresuid"
+                                             (function int (* uid-t) (* uid-t) (* uid-t)))
+                               (addr ruid) (addr euid) (addr suid))))
+          (if (minusp r)
+              (syscall-error 'getresuid)
+              (values ruid euid suid))))))
   (define-call "setegid" int minusp (gid gid-t))
   #-sunos
   (define-call "setfsgid" int minusp (gid gid-t))
@@ -696,6 +717,7 @@ not supported."
  (defun cfsetispeed (speed &optional termios)
    (declare (type (or null termios) termios))
    (with-alien-termios a-termios ()
+     (termios-to-alien termios a-termios)
      (let ((r (alien-funcall
                (extern-alien "cfsetispeed"
                              (function int (* alien-termios) speed-t))
@@ -710,6 +732,7 @@ not supported."
  (defun cfsetospeed (speed &optional termios)
    (declare (type (or null termios) termios))
    (with-alien-termios a-termios ()
+     (termios-to-alien termios a-termios)
      (let ((r (alien-funcall
                (extern-alien "cfsetospeed"
                              (function int (* alien-termios) speed-t))
