@@ -50,20 +50,21 @@
 (defun check-bound (array bound index)
   (declare (type index bound)
            (fixnum index))
-  (%check-bound array bound index))
+  (%check-bound array bound index)
+  index)
 
 (defun %with-array-data/fp (array start end)
   (%with-array-data-macro array start end :check-bounds t :check-fill-pointer t))
 
 (defun %with-array-data (array start end)
-  (%with-array-data-macro array start end :check-bounds t :check-fill-pointer nil))
+  (%with-array-data-macro array start end :check-bounds t :array-header-p t))
 
 (defun %data-vector-and-index (array index)
   (if (array-header-p array)
       (multiple-value-bind (vector index)
           (%with-array-data array index nil)
         (values vector index))
-      (values array index)))
+      (values (truly-the (simple-array * (*)) array) index)))
 
 ;;;; MAKE-ARRAY
 (defun %integer-vector-widetag-and-n-bits (signed high)
@@ -140,7 +141,9 @@
                                   sb!vm:*specialized-array-element-type-properties*
                                   :key #'sb!vm:saetp-typecode))))))
     (flet ((ill-type ()
-             (error "Invalid type specifier: ~s" type))
+             (declare (optimize allow-non-returning-tail-call))
+             (error "Invalid type specifier: ~/sb!impl:print-type-specifier/"
+                    type))
            (integer-interval-widetag (low high)
              (if (minusp low)
                  (%integer-vector-widetag-and-n-bits
@@ -523,7 +526,7 @@ of specialized arrays is supported."
   ;; is expensive, but probably worth the trouble as once we've allocated
   ;; the vector we have no way to get rid of it anymore...
   (when (eq t (upgraded-array-element-type element-type))
-    (error "Static arrays of type ~S not supported."
+    (error "Static arrays of type ~/sb!impl:print-type-specifier/ not supported."
            element-type))
   (validate-array-initargs initial-element-p initial-element
                            initial-contents-p initial-contents nil) ; for effect
@@ -534,11 +537,13 @@ of specialized arrays is supported."
              (length initial-contents)
              length))
     (unless (every (lambda (x) (typep x element-type)) initial-contents)
-      (error ":INITIAL-CONTENTS contains elements not of type ~S."
+      (error ":INITIAL-CONTENTS contains elements not of type ~
+               ~/sb!impl:print-type-specifier/."
              element-type)))
   (when initial-element-p
     (unless (typep initial-element element-type)
-      (error ":INITIAL-ELEMENT ~S is not of type ~S."
+      (error ":INITIAL-ELEMENT ~S is not of type ~
+               ~/sb!impl:print-type-specifier/."
              initial-element element-type)))
   ;; STEP 2
   ;;
@@ -785,6 +790,7 @@ of specialized arrays is supported."
 
 (declaim (ftype (function (array) nil) invalid-array-error))
 (defun invalid-array-error (array)
+  (declare (optimize allow-non-returning-tail-call))
   (aver (array-header-p array))
   ;; Array invalidation stashes the original dimensions here...
   (let ((dims (%array-displaced-p array))
@@ -799,6 +805,7 @@ of specialized arrays is supported."
 (declaim (ftype (function (array t integer &optional t) nil)
                 invalid-array-index-error))
 (defun invalid-array-index-error (array index bound &optional axis)
+  (declare (optimize allow-non-returning-tail-call))
   (if (invalid-array-p array)
       (invalid-array-error array)
       (error 'invalid-array-index-error
@@ -992,20 +999,26 @@ of specialized arrays is supported."
 (defun array-dimensions (array)
   #!+sb-doc
   "Return a list whose elements are the dimensions of the array"
-  (declare (array array))
-  (if (array-header-p array)
-      (do ((results nil (cons (array-dimension array index) results))
-           (index (1- (array-rank array)) (1- index)))
-          ((minusp index) results))
-      (list (array-dimension array 0))))
+  (declare (explicit-check))
+  (cond ((array-header-p array)
+         (do ((results nil (cons (%array-dimension array index) results))
+              (index (1- (%array-rank array)) (1- index)))
+             ((minusp index) results)))
+        ((typep array 'vector)
+         (list (length array)))
+        (t
+         (sb!c::%type-check-error/c array 'object-not-array-error))))
 
 (defun array-total-size (array)
   #!+sb-doc
   "Return the total number of elements in the Array."
-  (declare (array array))
-  (if (array-header-p array)
-      (%array-available-elements array)
-      (length (the vector array))))
+  (declare (explicit-check))
+  (cond ((array-header-p array)
+         (%array-available-elements array))
+        ((typep array 'vector)
+         (length array))
+        (t
+         (sb!c::%type-check-error/c array 'object-not-array-error))))
 
 (defun array-displacement (array)
   #!+sb-doc
@@ -1019,8 +1032,8 @@ of specialized arrays is supported."
 
 (defun adjustable-array-p (array)
   #!+sb-doc
-  "Return T if (ADJUST-ARRAY ARRAY...) would return an array identical
-   to the argument, this happens for complex arrays."
+  "Return T if and only if calling ADJUST-ARRAY on ARRAY will return
+   the identical object."
   (declare (array array))
   ;; Note that this appears not to be a fundamental limitation.
   ;; non-vector SIMPLE-ARRAYs are in fact capable of being adjusted,
@@ -1038,6 +1051,7 @@ of specialized arrays is supported."
   (and (array-header-p array) (%array-fill-pointer-p array)))
 
 (defun fill-pointer-error (vector &optional arg)
+  (declare (optimize allow-non-returning-tail-call))
   (cond (arg
          (aver (array-has-fill-pointer-p vector))
          (let ((max (%array-available-elements vector)))
@@ -1063,7 +1077,8 @@ of specialized arrays is supported."
       (fill-pointer-error vector)))
 
 (defun %set-fill-pointer (vector new)
-  (declare (explicit-check))
+  (declare (explicit-check)
+           (index new))
   (flet ((oops (x)
            (fill-pointer-error vector x)))
     (cond ((not (array-has-fill-pointer-p vector))
@@ -1094,21 +1109,73 @@ of specialized arrays is supported."
            (setf (%array-fill-pointer array) (1+ fill-pointer))
            fill-pointer))))
 
+;;; Widetags of FROM and TO should be equal
+(defun copy-vector-data (from to start end n-bits)
+  (declare (vector from to)
+           (index start end)
+           ((integer 0 255) n-bits))
+  (let ((from-length (length from)))
+    (cond ((simple-vector-p from)
+           (replace (truly-the simple-vector to)
+                    (truly-the simple-vector from)
+                    :start2 start :end2 end))
+          ;; Vector sizes are double-word aligned and have zeros in
+          ;; the extra word so it's safe to copy when the boundaries
+          ;; are matching the whole vector.
+          ;; A more generic routine is left for another time, even if
+          ;; only handling aligned data since it will avoid consing
+          ;; floats or word bignums.
+          ((and (= start 0)
+                (= end from-length))
+           (let ((words (ceiling (* from-length n-bits)
+                                 sb!vm:n-word-bits)))
+             (loop for i below words
+                   do (setf
+                       (%vector-raw-bits to i)
+                       (%vector-raw-bits from i)))))
+          (t
+           (replace to
+                    from
+                    :start2 start :end2 end)))
+    to))
+
+(defun extend-vector (vector min-extension)
+  (declare (optimize speed)
+           (vector vector))
+  (let* ((old-length (length vector))
+         (min-extension (or min-extension
+                            (min old-length
+                                 (- array-dimension-limit old-length))))
+         (new-length (the index (+ old-length
+                                   (max 1 min-extension))))
+         (fill-pointer (1+ old-length)))
+    (declare (fixnum new-length min-extension))
+    (with-array-data ((old-data vector) (old-start)
+                      (old-end old-length))
+      (let* ((widetag (%other-pointer-widetag old-data))
+             (n-bits (aref %%simple-array-n-bits%% widetag))
+             (new-data
+               (allocate-vector-with-widetag widetag new-length n-bits)))
+        (copy-vector-data old-data new-data old-start old-end n-bits)
+        (setf (%array-data-vector vector) new-data
+              (%array-available-elements vector) new-length
+              (%array-fill-pointer vector) fill-pointer
+              (%array-displacement vector) 0
+              (%array-dimension vector 0) new-length
+              (%array-displaced-p vector) nil)
+        vector))))
+
 (defun vector-push-extend (new-element vector &optional min-extension)
   (declare (type (or null (and index (integer 1))) min-extension))
   (declare (explicit-check))
-  (let ((fill-pointer (fill-pointer vector)))
-    (when (= fill-pointer (%array-available-elements vector))
-      (let ((min-extension
-              (or min-extension
-                  (let ((length (length vector)))
-                    (min length
-                         (- array-dimension-limit length))))))
-        (adjust-array vector (+ fill-pointer (max 1 min-extension)))))
+  (let* ((fill-pointer (fill-pointer vector))
+         (new-fill-pointer (1+ fill-pointer)))
+    (if (= fill-pointer (%array-available-elements vector))
+        (extend-vector vector min-extension)
+        (setf (%array-fill-pointer vector) new-fill-pointer))
     ;; disable bounds checking
     (locally (declare (optimize (safety 0)))
       (setf (aref vector fill-pointer) new-element))
-    (setf (%array-fill-pointer vector) (1+ fill-pointer))
     fill-pointer))
 
 (defun vector-pop (array)
@@ -1152,7 +1219,9 @@ of specialized arrays is supported."
            ;; This is weird. Should check upgraded type against actual
            ;; array element type I think. See lp#1331299. CLHS says that
            ;; "consequences are unspecified" so current behavior isn't wrong.
-           (error "The new element type, ~S, is incompatible with old type."
+           (error "The new element type, ~
+                    ~/sb!impl:print-type-specifier/, is incompatible ~
+                    with old type."
                   element-type))
           ((and fill-pointer (/= array-rank 1))
            (error "Only vectors can have fill pointers."))
@@ -1185,9 +1254,10 @@ of specialized arrays is supported."
              (unless (or (eql element-type (array-element-type displaced-to))
                          (subtypep element-type (array-element-type displaced-to)))
                ;; See lp#1331299 again. Require exact match on upgraded type?
-               (error "can't displace an array of type ~S into another of ~
-                       type ~S"
-                      element-type (array-element-type displaced-to)))
+               (error "can't displace an array of type ~
+                        ~/sb!impl:print-type-specifier/ into another ~
+                        of type ~/sb!impl:print-type-specifier/"
+                        element-type (array-element-type displaced-to)))
              (let ((displacement (or displaced-index-offset 0))
                    (array-size (apply #'* dimensions)))
                (declare (fixnum displacement array-size))
@@ -1460,7 +1530,8 @@ function to be removed without further warning."
          ;; would be a win...
          (unless (or (not initial-element-p)
                      (typep initial-element element-type))
-           (error "~S can't be used to initialize an array of type ~S."
+           (error "~S can't be used to initialize an array of type ~
+                    ~/sb!impl:print-type-specifier/."
                   initial-element element-type))
          (let ((temp (if initial-element-p
                          (make-array new-length :initial-element initial-element)
@@ -1665,3 +1736,14 @@ function to be removed without further warning."
                    (setf ,tag (%other-pointer-widetag ,(first args))))
                  (svref ,',table-name ,tag)))
              ,@args))))))
+
+(defun sb!kernel::check-array-shape (array dimensions)
+  (when (let ((dimensions dimensions))
+          (dotimes (i (array-rank array))
+            (unless (eql (array-dimension array i) (pop dimensions))
+              (return t))))
+    (error "malformed :INITIAL-CONTENTS: ~S should have dimensions ~S"
+           (make-array dimensions :displaced-to (%array-data-vector array)
+                                  :element-type (array-element-type array))
+           (array-dimensions array)))
+  array)
