@@ -282,6 +282,7 @@
 
 (defun add-bignums (a b)
   (declare (type bignum-type a b))
+  (declare (muffle-conditions compiler-note)) ; returns lispobj, so what.
   (let ((len-a (%bignum-length a))
         (len-b (%bignum-length b)))
     (multiple-value-bind (a len-a b len-b)
@@ -441,6 +442,7 @@
 
 (defun multiply-fixnums (a b)
   (declare (fixnum a b))
+  (declare (muffle-conditions compiler-note)) ; returns lispobj, so what.
   (let* ((a-minusp (minusp a))
          (b-minusp (minusp b)))
     (multiple-value-bind (high low)
@@ -523,8 +525,8 @@
   ;; The asserts in the GCD implementation are way too expensive to
   ;; check in normal use, and are disabled here.
   (sb!xc:defmacro gcd-assert (&rest args)
-    (if nil
-        `(assert ,@args)))
+    (declare (ignore args))
+    #+sb-bignum-assertions `(assert ,@args))
   ;; We'll be doing a lot of modular arithmetic.
   (sb!xc:defmacro modularly (form)
     `(logand all-ones-digit ,form)))
@@ -589,6 +591,7 @@
 ;;; paper, but uses some clever bit-twiddling nicked from Nickle to do it.
 (declaim (inline bmod))
 (defun bmod (u v)
+  (declare (muffle-conditions compiler-note)) ; returns lispobj, so what.
   (let ((ud (%bignum-ref u 0))
         (vd (%bignum-ref v 0))
         (umask 0)
@@ -939,6 +942,7 @@
 
 ;;; This assumes bignum is positive; that is, the result of negating it will
 ;;; stay in the provided allocated bignum.
+(declaim (maybe-inline negate-bignum-buffer-in-place))
 (defun negate-bignum-buffer-in-place (bignum bignum-len)
   (bignum-negate-loop bignum bignum-len bignum)
   bignum)
@@ -1307,6 +1311,7 @@
 (defun bignum-logcount (bignum)
   (declare (type bignum-type bignum)
            (optimize speed))
+  (declare (muffle-conditions compiler-note)) ; returns lispobj, so what.
   (let ((length (%bignum-length bignum))
         (result 0))
     (declare (type bignum-length length)
@@ -1465,6 +1470,38 @@
 ;;;; There used to be a bunch of code to implement "efficient" versions of LDB
 ;;;; and DPB here.  But it apparently was never used, so it's been deleted.
 ;;;;   --njf, 2007-02-04
+
+;; This could be used by way of a transform, though for now it's specifically
+;; a helper for %LDB in the limited case that it recognizes as non-consing.
+(defun ldb-bignum=>fixnum (byte-size byte-pos bignum)
+  (declare (type (integer 0 #.sb!vm:n-positive-fixnum-bits) byte-size)
+           (type bit-index byte-pos))
+  (multiple-value-bind (word-index bit-index) (floor byte-pos digit-size)
+    (let ((n-digits (%bignum-length bignum)))
+      (cond ((>= word-index n-digits) ; load from the infinitely extended sign word
+             (ldb (byte byte-size 0) (%sign-digit bignum n-digits)))
+            ((<= (+ bit-index byte-size) digit-size) ; contained in one word
+             ;; This case takes care of byte-size = 0 also.
+             (ldb (byte byte-size bit-index) (%bignum-ref bignum word-index)))
+            (t
+             ;; At least one bit is obtained from each of two words,
+             ;; and not more than two words.
+             (let* ((low-part-size
+                     (truly-the (integer 1 #.(1- sb!vm:n-positive-fixnum-bits))
+                                (- digit-size bit-index)))
+                    (high-part-size
+                     (truly-the (integer 1 #.(1- sb!vm:n-positive-fixnum-bits))
+                                (- byte-size low-part-size))))
+               (logior (truly-the (and fixnum unsigned-byte) ; high part
+                         (let ((word-index (1+ word-index)))
+                           (if (< word-index n-digits) ; next word exists
+                               (ash (ldb (byte high-part-size 0)
+                                         (%bignum-ref bignum word-index))
+                                    low-part-size)
+                               (mask-field (byte high-part-size low-part-size)
+                                           (%sign-digit bignum n-digits)))))
+                       (ldb (byte low-part-size bit-index) ; low part
+                            (%bignum-ref bignum word-index)))))))))
 
 ;;;; TRUNCATE
 
@@ -1551,6 +1588,7 @@
 ;;; reduction in readability that was introduced. --JES, 2004-08-07
 (defun bignum-truncate (x y)
   (declare (type bignum-type x y))
+  (declare (muffle-conditions compiler-note)) ; returns lispobj, so what.
   (let (truncate-x truncate-y)
     (labels
         ;;; Divide X by Y when Y is a single bignum digit. BIGNUM-TRUNCATE
@@ -1572,7 +1610,8 @@
                  ;; SHIFT-RIGHT-UNALIGNED won't do the right thing
                  ;; with a shift count of 0 or -1, so special case this.
                  (cond ((= y 0)
-                        (error 'division-by-zero))
+                        (error 'division-by-zero :operation 'truncate
+                               :operands (list x y)))
                        ((= y 1)
                         ;; We could probably get away with (VALUES X 0)
                         ;; here, but it's not clear that some of the
@@ -1899,7 +1938,8 @@
 (defun %normalize-bignum (result len)
   (declare (type bignum-type result)
            (type bignum-length len)
-           (inline %normalize-bignum-buffer))
+           (muffle-conditions compiler-note)
+           #!-sb-fluid (inline %normalize-bignum-buffer))
   (let ((newlen (%normalize-bignum-buffer result len)))
     (declare (type bignum-length newlen))
     (unless (= newlen len)
@@ -1918,7 +1958,7 @@
 (defun %mostly-normalize-bignum (result len)
   (declare (type bignum-type result)
            (type bignum-length len)
-           (inline %normalize-bignum-buffer))
+           #!-sb-fluid (inline %normalize-bignum-buffer))
   (let ((newlen (%normalize-bignum-buffer result len)))
     (declare (type bignum-length newlen))
     (unless (= newlen len)
