@@ -67,7 +67,9 @@
          (let ((bound (ash 1 s)))
            `(integer 0 ,(- bound bite 1))))
         (t
-         (error "Bad size specified for UNSIGNED-BYTE type specifier: ~S." s))))
+         (error "Bad size specified for UNSIGNED-BYTE type specifier: ~
+                  ~/sb!impl:print-type-specifier/."
+                s))))
 
 ;;; Motivated by the mips port. -- CSR, 2002-08-22
 (def!type signed-byte-with-a-bite-out (s bite)
@@ -76,7 +78,9 @@
          (let ((bound (ash 1 (1- s))))
            `(integer ,(- bound) ,(- bound bite 1))))
         (t
-         (error "Bad size specified for SIGNED-BYTE type specifier: ~S." s))))
+         (error "Bad size specified for SIGNED-BYTE type specifier: ~
+                  ~/sb!impl:print-type-specifier/."
+                s))))
 
 (def!type load/store-index (scale lowtag min-offset
                                  &optional (max-offset min-offset))
@@ -831,6 +835,7 @@
 ;;; PACKAGE-DESIGNATOR is actually a deleted package, and in that case
 ;;; you generally do want to signal an error instead of proceeding.)
 (defun %find-package-or-lose (package-designator)
+  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
   (or (find-package package-designator)
       (error 'simple-package-error
              :package package-designator
@@ -841,6 +846,7 @@
 ;;; consequences of most operations on deleted packages are
 ;;; unspecified. We try to signal errors in such cases.
 (defun find-undeleted-package-or-lose (package-designator)
+  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
   (let ((maybe-result (%find-package-or-lose package-designator)))
     (if (package-%name maybe-result)    ; if not deleted
         maybe-result
@@ -860,6 +866,7 @@
 
 ;;; Signal an error unless NAME is a legal function name.
 (defun legal-fun-name-or-type-error (name)
+  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
   (unless (legal-fun-name-p name)
     (error 'simple-type-error
            :datum name
@@ -1145,7 +1152,7 @@
                        (:test (setq test (second option)))
                        (t
                         (error "bad option: ~S" (first option)))))))))))
-    `(sb!xc:defmethod print-object ((structure ,name) ,stream)
+    `(defmethod print-object ((structure ,name) ,stream)
        (pprint-logical-block (,stream nil)
          (print-unreadable-object (structure
                                    ,stream
@@ -1163,6 +1170,28 @@
   ;; symbol will not be printed without a prefix.
   (let ((*package* *keyword-package*))
     (write symbol :stream stream :escape t)))
+
+(declaim (special sb!pretty:*pprint-quote-with-syntactic-sugar*))
+(defun print-type-specifier (stream type-specifier &optional colon at)
+  (declare (ignore colon at))
+  ;; Binding *PPRINT-QUOTE-WITH-SYNTACTIC-SUGAR* prevents certain
+  ;; [f]types from being printed unhelpfully:
+  ;;
+  ;;   (function ())           => #'NIL
+  ;;   (function *)            => #'*
+  ;;   (function (function a)) => #'#'A
+  ;;
+  ;; Binding *PACKAGE* to the COMMON-LISP package causes specifiers
+  ;; like CL:FUNCTION, CL:INTEGER, etc. to be printed without package
+  ;; prefix but forces printing with package prefix for other
+  ;; specifiers.
+  (let ((sb!pretty:*pprint-quote-with-syntactic-sugar* nil)
+        (*package* *cl-package*))
+    (prin1 type-specifier stream)))
+
+(defun print-type (stream type &optional colon at)
+  (print-type-specifier stream (type-specifier type) colon at))
+
 
 ;;;; etc.
 
@@ -1225,6 +1254,7 @@
               (deprecation-info-replacements info)))))
 
 (defun deprecation-error (software version namespace name replacements)
+  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
   (error 'deprecation-error
          :namespace namespace
          :name name
@@ -1354,24 +1384,23 @@
 
 (defun print-deprecation-replacements (stream replacements &optional colonp atp)
   (declare (ignore colonp atp))
+  ;; I don't think this is callable during cross-compilation, is it?
   (apply #'format stream
-         (!uncross-format-control
-          "~#[~;~
-             Use ~/sb!impl:print-symbol-with-prefix/ instead.~;~
-             Use ~/sb!impl:print-symbol-with-prefix/ or ~
-             ~/sb!impl:print-symbol-with-prefix/ instead.~:;~
+         "~#[~;~
+             Use ~/sb-impl:print-symbol-with-prefix/ instead.~;~
+             Use ~/sb-impl:print-symbol-with-prefix/ or ~
+             ~/sb-impl:print-symbol-with-prefix/ instead.~:;~
              Use~@{~#[~; or~] ~
-             ~/sb!impl:print-symbol-with-prefix/~^,~} instead.~
-           ~]")
+             ~/sb-impl:print-symbol-with-prefix/~^,~} instead.~
+           ~]"
          replacements))
 
 (defun print-deprecation-message (namespace name software version
                                   &optional replacements stream)
   (format stream
-          (!uncross-format-control
            "The ~(~A~) ~/sb!impl:print-symbol-with-prefix/ has been ~
             deprecated as of ~@[~A ~]version ~A.~
-            ~@[~2%~/sb!impl::print-deprecation-replacements/~]")
+            ~@[~2%~/sb!impl::print-deprecation-replacements/~]"
           namespace name software version replacements))
 
 (defun setup-function-in-final-deprecation
