@@ -68,7 +68,7 @@
   (define-fasl-format-features))
 
 ;;; the code for a character which terminates a fasl file header
-(def!constant +fasl-header-string-stop-char-code+ 255)
+(defconstant +fasl-header-string-stop-char-code+ 255)
 
 ;;; This value should be incremented when the system changes in such a
 ;;; way that it will no longer work reliably with old fasl files. In
@@ -76,7 +76,7 @@
 ;;; versions which break binary compatibility. But it certainly should
 ;;; be incremented for release versions which break binary
 ;;; compatibility.
-(def!constant +fasl-file-version+ 78)
+(defconstant +fasl-file-version+ 78)
 ;;; (description of versions before 0.9.0.1 deleted in 0.9.17)
 ;;; 56: (2005-05-22) Something between 0.9.0.1 and 0.9.0.14. My money is
 ;;;     on 0.9.0.6 (MORE CASE CONSISTENCY).
@@ -131,24 +131,24 @@
 
 ;;;; the FOP database
 
-(declaim (simple-vector *fop-names* *fop-funs*))
+(declaim (simple-vector **fop-names** **fop-funs**))
 
 ;;; a vector indexed by a FaslOP that yields the FOP's name
-(defvar *fop-names* (make-array 256 :initial-element nil))
+(defglobal **fop-names** (make-array 256 :initial-element nil))
 
 ;;; a vector indexed by a FaslOP that yields a function which performs
 ;;; the operation. Most functions take 0 arguments - they only manipulate
 ;;; the fop stack. But if the fop is defined to receive an argument (or two)
 ;;; then loader's main loop is responsible for supplying it.
-(defvar *fop-funs* (make-array 256 :initial-element 0))
+(defglobal **fop-funs** (make-array 256 :initial-element 0))
 
 ;; Two bitmaps indicate function signature. One tells whether the fop takes
 ;; operands (other than from the stack). Each consecutive block of 4 opcodes
 ;; either does or doesn't, so the array is 1/4th the size of the opcode space.
 ;; The other tells whether the fop wants its result pushed on the stack.
 (declaim (type (cons (simple-bit-vector 64) (simple-bit-vector 256))
-               *fop-signatures*))
-(defglobal *fop-signatures*
+               **fop-signatures**))
+(defglobal **fop-signatures**
     (cons (make-array 64 :element-type 'bit :initial-element 0)
           (make-array 256 :element-type 'bit :initial-element 0)))
 
@@ -159,8 +159,24 @@
   "the current number of recursive LOADs")
 (declaim (type index *load-depth*))
 
-;;; the FASL file we're reading from
-(defvar *fasl-input-stream*)
-(declaim (type ansi-stream *fasl-input-stream*))
+(defun make-fop-vector (size)
+  (declare (type index size))
+  (let ((vector (make-array size)))
+    (setf (aref vector 0) 0)
+    vector))
 
-(defvar *load-code-verbose* nil)
+;;; a holder for the FASL file we're reading from
+(defstruct (fasl-input (:conc-name %fasl-input-)
+                       (:constructor make-fasl-input (stream))
+                       (:predicate nil)
+                       (:copier nil))
+  (stream nil :type ansi-stream :read-only t)
+  (table (make-fop-vector 1000) :type simple-vector)
+  (stack (make-fop-vector 100) :type simple-vector)
+  (deprecated-stuff nil :type list)
+  ;; Sometimes we want to skip over any FOPs with side-effects (like
+  ;; function calls) while executing other FOPs. SKIP-UNTIL will
+  ;; either contain the position where the skipping will stop, or
+  ;; NIL if we're executing normally.
+  (skip-until nil))
+(declaim (freeze-type fasl-input))
