@@ -22,7 +22,7 @@
 ;;; Lisp call, and saves number consing to boot.
 (eval-when (:compile-toplevel :execute)
 
-(sb!xc:defmacro def-math-rtn (name num-args)
+(sb!xc:defmacro def-math-rtn (name num-args &optional wrapper)
   (let ((function (symbolicate "%" (string-upcase name)))
         (args (loop for i below num-args
                     collect (intern (format nil "ARG~D" i)))))
@@ -30,7 +30,7 @@
        (declaim (inline ,function))
        (defun ,function ,args
          (alien-funcall
-          (extern-alien ,name
+          (extern-alien ,(format nil "~:[~;sb_~]~a" wrapper name)
                         (function double-float
                                   ,@(loop repeat num-args
                                           collect 'double-float)))
@@ -59,7 +59,7 @@
   (def %log (x))
   (def %exp (x)))
 
-#!+x86-64 ;; for constant folding
+#!+(or x86-64 arm-vfp arm64) ;; for constant folding
 (macrolet ((def (name ll)
              `(defun ,name ,ll (,name ,@ll))))
   (def %sqrt (x)))
@@ -75,88 +75,32 @@
 #!-x86 (def-math-rtn "tan" 1)
 #!-x86 (def-math-rtn "atan" 1)
 #!-x86 (def-math-rtn "atan2" 2)
-#!-(and win32 x86)
-(progn
-  (def-math-rtn "acos" 1)
-  (def-math-rtn "asin" 1)
-  (def-math-rtn "cosh" 1)
-  (def-math-rtn "sinh" 1)
-  (def-math-rtn "tanh" 1)
-  #!-win32
-  (progn
-    (def-math-rtn "asinh" 1)
-    (def-math-rtn "acosh" 1)
-    (def-math-rtn "atanh" 1)))
-#!+win32
-(progn
-  #!-x86-64
-  (progn
-    (declaim (inline %asin))
-    (defun %asin (number)
-      (%atan (/ number (sqrt (- 1 (* number number))))))
-    (declaim (inline %acos))
-    (defun %acos (number)
-      (- (/ pi 2) (%asin number)))
-    (declaim (inline %cosh))
-    (defun %cosh (number)
-      (/ (+ (exp number) (exp (- number))) 2))
-    (declaim (inline %sinh))
-    (defun %sinh (number)
-      (/ (- (exp number) (exp (- number))) 2))
-    (declaim (inline %tanh))
-    (defun %tanh (number)
-      (/ (%sinh number) (%cosh number))))
-  (declaim (inline %asinh))
-  (defun %asinh (number)
-    (log (+ number (sqrt (+ (* number number) 1.0d0))) #.(exp 1.0d0)))
-  (declaim (inline %acosh))
-  (defun %acosh (number)
-    (log (+ number (sqrt (- (* number number) 1.0d0))) #.(exp 1.0d0)))
-  (declaim (inline %atanh))
-  (defun %atanh (number)
-    (let ((ratio (/ (+ 1 number) (- 1 number))))
-      ;; Were we effectively zero?
-      (if (= ratio -1.0d0)
-          0.0d0
-          (/ (log ratio #.(exp 1.0d0)) 2.0d0)))))
+
+(def-math-rtn "acos" 1 #!+win32 t)
+(def-math-rtn "asin" 1 #!+win32 t)
+(def-math-rtn "cosh" 1 #!+win32 t)
+(def-math-rtn "sinh" 1 #!+win32 t)
+(def-math-rtn "tanh" 1 #!+win32 t)
+(def-math-rtn "asinh" 1 #!+win32 t)
+(def-math-rtn "acosh" 1 #!+win32 t)
+(def-math-rtn "atanh" 1 #!+win32 t)
 
 ;;; exponential and logarithmic
+(def-math-rtn "hypot" 2 #!+win32 t)
 #!-x86 (def-math-rtn "exp" 1)
 #!-x86 (def-math-rtn "log" 1)
 #!-x86 (def-math-rtn "log10" 1)
-#!-(and win32 x86) (def-math-rtn "pow" 2)
-#!-(or x86 x86-64) (def-math-rtn "sqrt" 1)
-#!-win32 (def-math-rtn "hypot" 2)
+(def-math-rtn "pow" 2)
+#!-(or x86 x86-64 arm-vfp arm64) (def-math-rtn "sqrt" 1)
 #!-x86 (def-math-rtn "log1p" 1)
 
-#!+win32
-(progn
-  ;; This is written in a peculiar way to avoid overflow. Note that in
-  ;; sqrt(x^2 + y^2), either square or the sum can overflow.
-  ;;
-  ;; Factoring x^2 out of sqrt(x^2 + y^2) gives us the expression
-  ;; |x|sqrt(1 + (y/x)^2), which, assuming |x| >= |y|, can only overflow
-  ;; if |x| is sufficiently large.
-  ;;
-  ;; The ZEROP test suffices (y is non-negative) to guard against
-  ;; divisions by zero: x >= y > 0.
-  (declaim (inline %hypot))
-  (defun %hypot (x y)
-    (declare (type double-float x y))
-    (let ((x (abs x))
-          (y (abs y)))
-      (when (> y x)
-        (rotatef x y))
-      (if (zerop y)
-          x
-          (let ((y/x (/ y x)))
-            (* x (sqrt (1+ (* y/x y/x)))))))))
 
 ;;;; power functions
 
 (defun exp (number)
   #!+sb-doc
   "Return e raised to the power NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %exp number)
     ((complex)
@@ -199,6 +143,7 @@
 (defun expt (base power)
   #!+sb-doc
   "Return BASE raised to the POWER."
+  (declare (explicit-check))
   (if (zerop power)
     (if (and (zerop base) (floatp power))
         (error 'arguments-out-of-domain-error
@@ -398,6 +343,7 @@
 (defun log (number &optional (base nil base-p))
   #!+sb-doc
   "Return the logarithm of NUMBER in the base BASE, which defaults to e."
+  (declare (explicit-check))
   (if base-p
       (cond
         ((zerop base)
@@ -446,6 +392,7 @@
 (defun sqrt (number)
   #!+sb-doc
   "Return the square root of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (((foreach fixnum bignum ratio))
      (if (minusp number)
@@ -464,6 +411,7 @@
 (defun abs (number)
   #!+sb-doc
   "Return the absolute value of the number."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (((foreach single-float double-float fixnum rational))
      (abs number))
@@ -475,10 +423,10 @@
           (sqrt (+ (* rx rx) (* ix ix))))
          (single-float
           (coerce (%hypot (coerce rx 'double-float)
-                          (coerce ix 'double-float))
+                          (coerce (truly-the single-float ix) 'double-float))
                   'single-float))
          (double-float
-          (%hypot rx ix)))))))
+          (%hypot rx (truly-the double-float ix))))))))
 
 (defun phase (number)
   #!+sb-doc
@@ -486,6 +434,7 @@
   For complex numbers, this is (atan (imagpart number) (realpart number)).
   For non-complex positive numbers, this is 0. For non-complex negative
   numbers this is PI."
+  (declare (explicit-check))
   (etypecase number
     (rational
      (if (minusp number)
@@ -505,6 +454,7 @@
 (defun sin (number)
   #!+sb-doc
   "Return the sine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %sin number)
     ((complex)
@@ -516,6 +466,7 @@
 (defun cos (number)
   #!+sb-doc
   "Return the cosine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %cos number)
     ((complex)
@@ -527,6 +478,7 @@
 (defun tan (number)
   #!+sb-doc
   "Return the tangent of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %tan number)
     ((complex)
@@ -535,12 +487,13 @@
 (defun cis (theta)
   #!+sb-doc
   "Return cos(Theta) + i sin(Theta), i.e. exp(i Theta)."
-  (declare (type real theta))
+  (declare (type real theta) (explicit-check))
   (complex (cos theta) (sin theta)))
 
 (defun asin (number)
   #!+sb-doc
   "Return the arc sine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     ((rational)
      (if (or (> number 1) (< number -1))
@@ -558,6 +511,7 @@
 (defun acos (number)
   #!+sb-doc
   "Return the arc cosine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     ((rational)
      (if (or (> number 1) (< number -1))
@@ -575,6 +529,7 @@
 (defun atan (y &optional (x nil xp))
   #!+sb-doc
   "Return the arc tangent of Y if X is omitted or Y/X if X is supplied."
+  (declare (explicit-check))
   (if xp
       (flet ((atan2 (y x)
                (declare (type double-float y x)
@@ -611,6 +566,7 @@
 (defun sinh (number)
   #!+sb-doc
   "Return the hyperbolic sine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %sinh number)
     ((complex)
@@ -622,6 +578,7 @@
 (defun cosh (number)
   #!+sb-doc
   "Return the hyperbolic cosine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %cosh number)
     ((complex)
@@ -633,6 +590,7 @@
 (defun tanh (number)
   #!+sb-doc
   "Return the hyperbolic tangent of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %tanh number)
     ((complex)
@@ -641,6 +599,7 @@
 (defun asinh (number)
   #!+sb-doc
   "Return the hyperbolic arc sine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %asinh number)
     ((complex)
@@ -649,6 +608,7 @@
 (defun acosh (number)
   #!+sb-doc
   "Return the hyperbolic arc cosine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     ((rational)
      ;; acosh is complex if number < 1
@@ -666,6 +626,7 @@
 (defun atanh (number)
   #!+sb-doc
   "Return the hyperbolic arc tangent of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     ((rational)
      ;; atanh is complex if |number| > 1
@@ -721,7 +682,7 @@
 ;;;;   complex-tan
 ;;;;
 ;;;; utility functions:
-;;;;   scalb logb
+;;;;   logb
 ;;;;
 ;;;; internal functions:
 ;;;;    square coerce-to-complex-type cssqs complex-log-scaled
@@ -765,20 +726,12 @@
   (declare (double-float x))
   (* x x))
 
-;;; original CMU CL comment, apparently re. SCALB and LOGB and
+;;; original CMU CL comment, apparently re. LOGB and
 ;;; perhaps CSSQS:
 ;;;   If you have these functions in libm, perhaps they should be used
 ;;;   instead of these Lisp versions. These versions are probably good
 ;;;   enough, especially since they are portable.
 
-;;; Compute 2^N * X without computing 2^N first. (Use properties of
-;;; the underlying floating-point format.)
-(declaim (inline scalb))
-(defun scalb (x n)
-  (declare (type double-float x)
-           (type double-float-exponent n))
-  (scale-float x n))
-
 ;;; This is like LOGB, but X is not infinity and non-zero and not a
 ;;; NaN, so we can always return an integer.
 (declaim (inline logb-finite))
@@ -836,6 +789,7 @@
 #!+long-float (eval-when (:compile-toplevel :load-toplevel :execute)
                 (error "needs work for long float support"))
 (defun cssqs (z)
+  (declare (muffle-conditions t))
   (let ((x (float (realpart z) 1d0))
         (y (float (imagpart z) 1d0)))
     ;; Would this be better handled using an exception handler to
@@ -872,8 +826,8 @@
               ;; least one is non-zero.. Thus logb returns a nice
               ;; integer.
               (let ((k (- (logb-finite (max (abs x) (abs y))))))
-                (values (+ (square (scalb x k))
-                           (square (scalb y k)))
+                (values (+ (square (scale-float x k))
+                           (square (scale-float y k)))
                         (- k))))
              (t
               (values rho 0)))))))
@@ -897,14 +851,11 @@
           (y (float (imagpart z) 1.0d0))
           (eta 0d0)
           (nu 0d0))
-      (declare (double-float x y eta nu))
-
-      (locally
-         ;; space 0 to get maybe-inline functions inlined.
-         (declare (optimize (speed 3) (space 0)))
-
+      (declare (double-float x y eta nu)
+               ;; get maybe-inline functions inlined.
+               (optimize (space 0)))
       (if (not (float-nan-p x))
-          (setf rho (+ (scalb (abs x) (- k)) (sqrt rho))))
+          (setf rho (+ (scale-float (abs x) (- k)) (sqrt rho))))
 
       (cond ((oddp k)
              (setf k (ash k -1)))
@@ -912,23 +863,24 @@
              (setf k (1- (ash k -1)))
              (setf rho (+ rho rho))))
 
-      (setf rho (scalb (sqrt rho) k))
+      (setf rho (scale-float (sqrt rho) k))
 
       (setf eta rho)
       (setf nu y)
 
       (when (/= rho 0d0)
-            (when (not (float-infinity-p (abs nu)))
-                  (setf nu (/ (/ nu rho) 2d0)))
-            (when (< x 0d0)
-                  (setf eta (abs nu))
-                  (setf nu (float-sign y rho))))
-       (coerce-to-complex-type eta nu z)))))
+        (when (not (float-infinity-p (abs nu)))
+          (setf nu (/ (/ nu rho) 2d0)))
+        (when (< x 0d0)
+          (setf eta (abs nu))
+          (setf nu (float-sign y rho))))
+      (coerce-to-complex-type eta nu z))))
 
 ;;; Compute log(2^j*z).
 ;;;
 ;;; This is for use with J /= 0 only when |z| is huge.
 (defun complex-log-scaled (z j)
+  (declare (muffle-conditions t))
   (declare (type (or rational complex) z)
            (fixnum j))
   ;; The constants t0, t1, t2 should be evaluated to machine
@@ -985,6 +937,7 @@
 ;;; i*y is never 0 since we have positive and negative zeroes. -- rtoy
 ;;; Compute atanh z = (log(1+z) - log(1-z))/2.
 (defun complex-atanh (z)
+  (declare (muffle-conditions t))
   (declare (type (or rational complex) z))
   (let* (;; constants
          (theta (/ (sqrt most-positive-double-float) 4.0d0))
@@ -1042,6 +995,7 @@
 
 ;;; Compute tanh z = sinh z / cosh z.
 (defun complex-tanh (z)
+  (declare (muffle-conditions t))
   (declare (type (or rational complex) z))
   (let ((x (float (realpart z) 1.0d0))
         (y (float (imagpart z) 1.0d0)))
