@@ -170,7 +170,7 @@
   (def-type-predicate-wrapper vectorp)
   (def-type-predicate-wrapper vector-nil-p))
 
-#!+(or x86 x86-64 arm)
+#!+(or x86 x86-64 arm arm64)
 (defun fixnum-mod-p (x limit)
   (and (fixnump x)
        (<= 0 x limit)))
@@ -181,6 +181,35 @@
        (member (%other-pointer-widetag x) choices)
        t))
 
+;;; Return the layout for an object. This is the basic operation for
+;;; finding out the "type" of an object, and is used for generic
+;;; function dispatch. The standard doesn't seem to say as much as it
+;;; should about what this returns for built-in objects. For example,
+;;; it seems that we must return NULL rather than LIST when X is NIL
+;;; so that GF's can specialize on NULL.
+(declaim (inline layout-of))
+#-sb-xc-host
+(defun layout-of (x)
+  (declare (optimize (speed 3) (safety 0)))
+  (cond ((%instancep x) (%instance-layout x))
+        ((funcallable-instance-p x) (%funcallable-instance-layout x))
+        ;; Compiler can dump literal layouts, which handily sidesteps
+        ;; the question of when cold-init runs L-T-V forms.
+        ((null x) #.(find-layout 'null))
+        (t
+         ;; Note that WIDETAG-OF is slightly suboptimal here and could be
+         ;; improved - we've already ruled out some of the lowtags.
+         (svref (load-time-value sb!kernel::**built-in-class-codes** t)
+                (widetag-of x)))))
+
+(declaim (inline classoid-of))
+#-sb-xc-host
+(defun classoid-of (object)
+  #!+sb-doc
+  "Return the class of the supplied object, which may be any Lisp object, not
+   just a CLOS STANDARD-OBJECT."
+  (layout-classoid (layout-of object)))
+
 ;;; Return the specifier for the type of object. This is not simply
 ;;; (TYPE-SPECIFIER (CTYPE-OF OBJECT)) because CTYPE-OF has different
 ;;; goals than TYPE-OF. In particular, speed is more important than
@@ -189,6 +218,7 @@
 (defun type-of (object)
   #!+sb-doc
   "Return the type of OBJECT."
+  (declare (explicit-check))
   ;; We have special logic for everything except arrays.
   ;; Arrays use CTYPE-OF and then convert the answer to a specifier.
   (typecase object
@@ -201,17 +231,26 @@
      (if (>= object 0)
          '(integer #.(1+ sb!xc:most-positive-fixnum))
          'bignum))
-    (standard-char 'standard-char)
-    (base-char 'base-char)
-    (extended-char 'extended-char)
-    ((member t) 'boolean)
-    (keyword 'keyword)
+    (character
+     (typecase object
+       (standard-char 'standard-char)
+       (base-char 'base-char)
+       (extended-char 'extended-char)))
+    ;; We "have to" (or have chosen to) pick off KEYWORD and BOOLEAN,
+    ;; so we may as well have a branch that returns early for any SYMBOL
+    ;; rather than falling into the CLASSOID-based test. But then since we
+    ;; do that, we also have to pick off NIL so that it doesn't say SYMBOL.
+    (symbol
+     (cond ((eq object t) 'boolean)
+           ((eq object nil) 'null)
+           ((eq (symbol-package object) *keyword-package*) 'keyword)
+           (t 'symbol)))
     ((or array complex #!+sb-simd-pack simd-pack)
      (let ((sb!kernel::*unparse-allow-negation* nil))
        (declare (special sb!kernel::*unparse-allow-negation*)) ; forward ref
        (type-specifier (ctype-of object))))
     (t
-     (let* ((classoid (layout-classoid (layout-of object)))
+     (let* ((classoid (classoid-of object))
             (name (classoid-name classoid)))
        (if (%instancep object)
            (case name
@@ -237,7 +276,8 @@
 ;;; IR1 might potentially transform EQL into %EQL/INTEGER.
 #!+integer-eql-vop
 (defun %eql/integer (obj1 obj2)
-  (eql (the integer obj1) (the integer obj2)))
+  ;; This is just for constand folding, no need to transform into the %EQL/INTEGER VOP
+  (eql obj1 obj2))
 
 (declaim (inline %eql))
 (defun %eql (obj1 obj2)
@@ -288,12 +328,15 @@
          (bit-vector-= x y))            ; DEFTRANSFORM
         (t
          (and (= (length x) (length y))
-              (do ((i 0 (1+ i))
-                   (length (length x)))
-                  ((= i length) t)
-                (declare (fixnum i))
-                (unless (= (bit x i) (bit y i))
-                  (return nil)))))))
+              (with-array-data ((x x) (start-x) (end-x) :force-inline t
+                                                        :check-fill-pointer t)
+                (with-array-data ((y y) (start-y) (end-y) :force-inline t
+                                                          :check-fill-pointer t)
+                  (declare (ignore end-y))
+                  (loop for x-i fixnum from start-x below end-x
+                        for y-i fixnum from start-y
+                        always (or (= (sbit x x-i)
+                                      (sbit y y-i))))))))))
 
 (defun equal (x y)
   #!+sb-doc
@@ -339,6 +382,81 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
                       x)
              t))))
 
+(defun instance-equalp (x y)
+  (let ((layout-x (%instance-layout x)))
+    (and
+     (eq layout-x (%instance-layout y))
+     ;; TODO: store one bit in the layout indicating whether EQUALP
+     ;; should scan slots. (basically a STRUCTURE-CLASSOID-P bit)
+     (structure-classoid-p (layout-classoid layout-x))
+     (macrolet ((slot-ref-equalp ()
+                  `(let ((x-el (%instance-ref x i))
+                         (y-el (%instance-ref y i)))
+                     (or (eq x-el y-el) (equalp x-el y-el)))))
+       (let ((bitmap (layout-bitmap layout-x)))
+         (if (zerop bitmap)
+             (loop for i of-type index from sb!vm:instance-data-start
+                   below (layout-length layout-x)
+                   always (slot-ref-equalp))
+             (let ((comparators (layout-equalp-tests layout-x)))
+               (unless (= (length comparators)
+                          (- (layout-length layout-x) sb!vm:instance-data-start))
+                 (bug "EQUALP got incomplete instance layout"))
+               ;; See remark at the source code for %TARGET-DEFSTRUCT
+               ;; explaining how to use the vector of comparators.
+               (loop for i of-type index from sb!vm:instance-data-start
+                     below (layout-length layout-x)
+                     for test = (data-vector-ref
+                                 comparators (- i sb!vm:instance-data-start))
+                     always (cond ((eql test 0) (slot-ref-equalp))
+                                  ((functionp test)
+                                   (funcall test i x y))
+                                  (t))))))))))
+
+;;; Doesn't work on simple vectors
+(defun array-equal-p (x y)
+  (declare (array x y))
+  (let ((rank (array-rank x)))
+    (= rank (array-rank y))
+    (dotimes (axis rank t)
+      (unless (= (%array-dimension x axis)
+                 (%array-dimension y axis))
+        (return nil)))
+    (with-array-data ((x x) (start-x) (end-x) :force-inline t
+                                              :array-header-p t)
+      (with-array-data ((y y) (start-y) (end-y) :force-inline t
+                                                :array-header-p t)
+        (declare (ignore end-y))
+        (let* ((reffers %%data-vector-reffers%%)
+               (getter-x (truly-the function (svref reffers (%other-pointer-widetag x))))
+               (getter-y (truly-the function (svref reffers (%other-pointer-widetag y)))))
+          (loop for x-i fixnum from start-x below end-x
+                for y-i fixnum from start-y
+                for x-el = (funcall getter-x x x-i)
+                for y-el = (funcall getter-y y y-i)
+                always (or (eq x-el y-el)
+                           (equalp x-el y-el))))))))
+
+(defun vector-equalp (x y)
+  (declare (vector x y))
+  (let ((length (length x)))
+    (and (vectorp y)
+         (= length (length y))
+         (with-array-data ((x x) (start-x) (end-x) :force-inline t
+                                                   :check-fill-pointer t)
+           (with-array-data ((y y) (start-y) (end-y) :force-inline t
+                                                     :check-fill-pointer t)
+             (declare (ignore end-y))
+             (let* ((reffers %%data-vector-reffers%%)
+                    (getter-x (truly-the function (svref reffers (%other-pointer-widetag x))))
+                    (getter-y (truly-the function (svref reffers (%other-pointer-widetag y)))))
+               (loop for x-i fixnum from start-x below end-x
+                     for y-i fixnum from start-y
+                     for x-el = (funcall getter-x x x-i)
+                     for y-el = (funcall getter-y y y-i)
+                     always (or (eq x-el y-el)
+                                (equalp x-el y-el)))))))))
+
 (defun equalp (x y)
   #+nil ; KLUDGE: If doc string, should be accurate: Talk about structures
   ; and HASH-TABLEs.
@@ -360,67 +478,17 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
          (and (hash-table-p y)
               (hash-table-equalp x y)))
         ((%instancep x)
-         (let ((layout-x (%instance-layout x)))
-           (and (%instancep y)
-                (eq layout-x (%instance-layout y))
-                ;; TODO: store one bit in the layout indicating whether EQUALP
-                ;; should scan slots. (basically a STRUCTURE-CLASSOID-P bit)
-                (structure-classoid-p (layout-classoid layout-x))
-                (macrolet ((slot-ref-equalp ()
-                             `(let ((x-el (%instance-ref x i))
-                                    (y-el (%instance-ref y i)))
-                                (or (eq x-el y-el) (equalp x-el y-el)))))
-                  #!-interleaved-raw-slots
-                  (let ((raw-len (layout-n-untagged-slots layout-x))
-                        (total-len (layout-length layout-x)))
-                    (and (dotimes (i (- total-len raw-len) t)
-                           (unless (slot-ref-equalp)
-                             (return nil)))
-                         (or (zerop raw-len)
-                             (raw-instance-slots-equalp layout-x x y))))
-                  #!+interleaved-raw-slots
-                  (let ((metadata (layout-untagged-bitmap layout-x)))
-                    (if (zerop metadata)
-                        (loop for i of-type index from sb!vm:instance-data-start
-                              below (layout-length layout-x)
-                              always (slot-ref-equalp))
-                        (let ((comparators (layout-equalp-tests layout-x)))
-                          (unless (= (length comparators)
-                                     (- (layout-length layout-x) sb!vm:instance-data-start))
-                            (bug "EQUALP got incomplete instance layout"))
-                          ;; See remark at the source code for %TARGET-DEFSTRUCT
-                          ;; explaining how to use the vector of comparators.
-                          (loop for i of-type index from sb!vm:instance-data-start
-                                below (layout-length layout-x)
-                                for test = (data-vector-ref
-                                            comparators (- i sb!vm:instance-data-start))
-                                always (cond ((eql test 0) (slot-ref-equalp))
-                                             ((functionp test)
-                                              (funcall test i x y))
-                                             (t))))))))))
+         (and (%instancep y)
+              (instance-equalp x y)))
+        ((and (bit-vector-p x)
+              (bit-vector-p y))
+         (bit-vector-= x y))
         ((vectorp x)
-         (let ((length (length x)))
-           (and (vectorp y)
-                (= length (length y))
-                (dotimes (i length t)
-                  (let ((x-el (aref x i))
-                        (y-el (aref y i)))
-                    (unless (or (eq x-el y-el)
-                                (equalp x-el y-el))
-                      (return nil)))))))
+         (and (vectorp y)
+              (vector-equalp x y)))
         ((arrayp x)
          (and (arrayp y)
-              (= (array-rank x) (array-rank y))
-              (dotimes (axis (array-rank x) t)
-                (unless (= (array-dimension x axis)
-                           (array-dimension y axis))
-                  (return nil)))
-              (dotimes (index (array-total-size x) t)
-                (let ((x-el (row-major-aref x index))
-                      (y-el (row-major-aref y index)))
-                  (unless (or (eq x-el y-el)
-                              (equalp x-el y-el))
-                    (return nil))))))
+              (array-equal-p x y)))
         (t nil)))
 
 (/show0 "about to do test cases in pred.lisp")
