@@ -170,13 +170,15 @@
   %status             ; either :RUNNING, :STOPPED, :EXITED, or :SIGNALED
   %exit-code          ; either exit code or signal
   core-dumped         ; T if a core image was dumped
-  #-win32 pty                 ; stream to child's pty, or NIL
+  #-win32 pty         ; stream to child's pty, or NIL
   input               ; stream to child's input, or NIL
   output              ; stream from child's output, or NIL
   error               ; stream from child's error output, or NIL
   status-hook         ; closure to call when PROC changes status
   plist               ; a place for clients to stash things
-  cookie)             ; list of the number of pipes from the subproc
+  cookie              ; list of the number of pipes from the subproc
+  #+win32
+  copiers)            ; list of sb-win32::io-copier
 
 (defmethod print-object ((process process) stream)
   (print-unreadable-object (process stream :type t)
@@ -186,13 +188,6 @@
          (format stream "~S ~S" (process-pid process) status)))
     process))
 
-#+sb-doc
-(setf (documentation 'process-p 'function)
-      "T if OBJECT is a PROCESS, NIL otherwise.")
-
-#+sb-doc
-(setf (documentation 'process-pid 'function) "The pid of the child process.")
-
 #+win32
 (define-alien-routine ("GetExitCodeProcess" get-exit-code-process)
     int
@@ -214,36 +209,24 @@
 
 #+sb-doc
 (setf (documentation 'process-exit-code 'function)
-      "The exit code or the signal of a stopped process.")
-
-#+sb-doc
-(setf (documentation 'process-core-dumped 'function)
-      "T if a core image was dumped by the process.")
-
-#+sb-doc
-(setf (documentation 'process-pty 'function)
-      "The pty stream of the process or NIL.")
-
-#+sb-doc
-(setf (documentation 'process-input 'function)
-      "The input stream of the process or NIL.")
-
-#+sb-doc
-(setf (documentation 'process-output 'function)
-      "The output stream of the process or NIL.")
-
-#+sb-doc
-(setf (documentation 'process-error 'function)
-      "The error stream of the process or NIL.")
-
-#+sb-doc
-(setf (documentation 'process-status-hook  'function)
-      "A function that is called when PROCESS changes its status.
-The function is called with PROCESS as its only argument.")
-
-#+sb-doc
-(setf (documentation 'process-plist  'function)
-      "A place for clients to stash things.")
+      "The exit code or the signal of a stopped process."
+      (documentation 'process-core-dumped 'function)
+      "T if a core image was dumped by the process."
+      (documentation 'process-pty 'function)
+      "The pty stream of the process or NIL."
+      (documentation 'process-input 'function)
+      "The input stream of the process or NIL."
+      (documentation 'process-output 'function)
+      "The output stream of the process or NIL."
+      (documentation 'process-error 'function)
+      "The error stream of the process or NIL."
+      (documentation 'process-status-hook 'function) "A function that is called when PROCESS changes its status.
+The function is called with PROCESS as its only argument."
+      (documentation 'process-plist 'function)
+      "A place for clients to stash things."
+      (documentation 'process-p 'function)
+      "T if OBJECT is a PROCESS, NIL otherwise."
+      (documentation 'process-pid 'function) "The pid of the child process.")
 
 (defun process-wait (process &optional check-for-stopped)
   #+sb-doc
@@ -252,13 +235,7 @@ CHECK-FOR-STOPPED is T, also returns when PROCESS is stopped. Returns
 PROCESS."
   (declare (ignorable check-for-stopped))
   #+win32
-  (let ((pid (process-pid process)))
-    (when (and pid (plusp pid))
-      (without-interrupts
-        (do ()
-            ((= 0
-                (with-local-interrupts
-                  (sb-win32:wait-object-or-signal pid))))))))
+  (sb-win32::win32-process-wait process)
   #-win32
   (loop
       (case (process-status process)
@@ -374,7 +351,8 @@ status slot."
                            (when pid
                              (multiple-value-bind (ok code)
                                  (sb-win32::get-exit-code-process pid)
-                               (when (and (plusp ok) (/= code 259))
+                               (when (and (plusp ok)
+                                          (/= code sb-win32::still-active))
                                  (setf (process-%status proc) :exited
                                        (process-%exit-code proc) code)
                                  (when (process-status-hook proc)
@@ -398,9 +376,7 @@ status slot."
 ;;; list of file descriptors to close when RUN-PROGRAM returns in the parent
 (defvar *close-in-parent* nil)
 
-;;; list of handlers installed by RUN-PROGRAM.  FIXME: nothing seems
-;;; to set this.
-#-win32
+;;; list of handlers installed by RUN-PROGRAM.
 (defvar *handlers-installed* nil)
 
 ;;; Find an unused pty. Return three values: the file descriptor for
@@ -785,9 +761,7 @@ Users Manual for details about the PROCESS structure.
          ;; communicate cleanup info.
          *close-on-error*
          *close-in-parent*
-         ;; Some other binding used only on non-Win32.  FIXME:
-         ;; nothing seems to set this.
-         #-win32 *handlers-installed*
+         *handlers-installed*
          ;; Establish PROC at this level so that we can return it.
          proc
          (progname (native-namestring program))
@@ -872,15 +846,9 @@ Users Manual for details about the PROCESS structure.
                                   :status-hook status-hook
                                   :cookie cookie
                                   #-win32 :pty #-win32 pty-stream
-                                  :%status #-win32 :running
-                                           #+win32 (if wait
-                                                       :exited
-                                                       :running)
-                                  :pid #-win32 child
-                                       #+win32 (if wait
-                                                   nil
-                                                   child)
-                                  #+win32 :%exit-code #+win32 (and wait child)))
+                                  :%status :running
+                                  :pid child
+                                  #+win32 :copiers #+win32 *handlers-installed*))
                            (push proc *active-processes*))))
                      ;; Report the error outside the lock.
                      (case child
@@ -901,10 +869,10 @@ Users Manual for details about the PROCESS structure.
         #-win32
         (dolist (handler *handlers-installed*)
           (remove-fd-handler handler)))
-      #-win32
       (when (and wait proc)
         (unwind-protect
              (process-wait proc)
+          #-win32
           (dolist (handler *handlers-installed*)
             (remove-fd-handler handler)))))
     proc))
@@ -912,6 +880,7 @@ Users Manual for details about the PROCESS structure.
 ;;; Install a handler for any input that shows up on the file
 ;;; descriptor. The handler reads the data and writes it to the
 ;;; stream.
+#-win32
 (defun copy-descriptor-to-stream (descriptor stream cookie external-format)
   (incf (car cookie))
   (let* ((handler nil)
@@ -1007,9 +976,16 @@ Users Manual for details about the PROCESS structure.
                     (t
                      (incf read-end count)
                      (funcall copy-fun))))))))
-    #-win32
     (push handler *handlers-installed*)))
 
+#+win32
+(defun copy-descriptor-to-stream (descriptor stream cookie external-format)
+  (declare (ignore cookie))
+  (push (sb-win32::make-io-copier :pipe descriptor
+                                  :stream stream
+                                  :external-format external-format)
+        *handlers-installed*))
+
 ;;; FIXME: something very like this is done in SB-POSIX to treat
 ;;; streams as file descriptor designators; maybe we can combine these
 ;;; two?  Additionally, as we have a couple of user-defined streams
@@ -1063,6 +1039,8 @@ Users Manual for details about the PROCESS structure.
                (unless fd
                  (fail "could not open a temporary file: ~A"
                        (strerror name/errno)))
+               #+win32
+               (setf (sb-win32::inheritable-handle-p fd) t)
                ;; Can't unlink an open file on Windows
                #-win32
                (unless (sb-unix:unix-unlink name/errno)
@@ -1084,7 +1062,8 @@ Users Manual for details about the PROCESS structure.
                                       (:input sb-unix:o_rdonly)
                                       (:output sb-unix:o_wronly)
                                       (t sb-unix:o_rdwr))
-                                    #o666)
+                                    #o666
+                                    #+win32 :overlapped #+win32 nil)
                (unless fd
                  (fail "~@<couldn't open ~S: ~2I~_~A~:>"
                        dev-null (strerror errno)))
@@ -1127,7 +1106,9 @@ Users Manual for details about the PROCESS structure.
              ;; GET-DESCRIPTOR-FOR uses &allow-other-keys, so rather
              ;; than munge the &rest list for OPEN, just disable keyword
              ;; validation there.
-             (with-open-stream (file (apply #'open object :allow-other-keys t
+             (with-open-stream (file (apply #'open object
+                                            :allow-other-keys t
+                                            #+win32 :overlapped #+win32 nil
                                             keys))
                (when file
                  (multiple-value-bind
@@ -1224,9 +1205,12 @@ Users Manual for details about the PROCESS structure.
                   (when fd
                     (return (values fd stream))))
                 (multiple-value-bind (read-fd write-fd)
-                    (sb-unix:unix-pipe)
+                    #-win32 (sb-unix:unix-pipe)
+                    #+win32 (sb-win32::make-named-pipe)
                   (unless read-fd
                     (fail "couldn't create pipe: ~S" (strerror write-fd)))
+                  #+win32
+                  (setf (sb-win32::inheritable-handle-p write-fd) t)
                   (copy-descriptor-to-stream read-fd object cookie
                                              external-format)
                   (push read-fd *close-on-error*)
