@@ -768,6 +768,9 @@
      (finish-output stream))
     (:element-type
      (stream-element-type stream))
+    (:element-mode
+     (stream-element-type-stream-element-mode
+      (stream-element-type stream)))
     (:stream-external-format
      (stream-external-format stream))
     (:interactive-p
@@ -782,6 +785,18 @@
      (file-string-length stream arg1))
     (:file-position
      (file-position stream arg1))))
+
+(declaim (inline stream-element-mode))
+(defun stream-element-mode (stream)
+  (declare (type stream stream))
+  (cond
+    ((fd-stream-p stream)
+     (fd-stream-element-mode stream))
+    ((and (ansi-stream-p stream)
+          (funcall (ansi-stream-misc stream) stream :element-mode)))
+    (t
+     (stream-element-type-stream-element-mode
+      (stream-element-type stream)))))
 
 ;;;; broadcast streams
 
@@ -845,6 +860,9 @@
          (if last
              (stream-element-type (car last))
              t)))
+      (:element-mode
+       (awhen (last streams)
+         (stream-element-mode (car it))))
       (:external-format
        (let ((last (last streams)))
          (if last
@@ -882,7 +900,7 @@
 
 ;;;; synonym streams
 
-(def!method print-object ((x synonym-stream) stream)
+(defmethod print-object ((x synonym-stream) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (format stream ":SYMBOL ~S" (synonym-stream-symbol x))))
 
@@ -999,7 +1017,13 @@
        (let ((in-type (stream-element-type in))
              (out-type (stream-element-type out)))
          (if (equal in-type out-type)
-             in-type `(and ,in-type ,out-type))))
+             in-type
+             `(and ,in-type ,out-type))))
+      (:element-mode
+       (let ((in-mode (stream-element-mode in))
+             (out-mode (stream-element-mode out)))
+         (when (equal in-mode out-mode)
+           in-mode)))
       (:close
        (set-closed-flame stream))
       (t
@@ -1026,7 +1050,7 @@
 
 (declaim (freeze-type concatenated-stream))
 
-(def!method print-object ((x concatenated-stream) stream)
+(defmethod print-object ((x concatenated-stream) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (format stream
             ":STREAMS ~S"
@@ -1123,7 +1147,7 @@
 
 (declaim (freeze-type echo-stream))
 
-(def!method print-object ((x echo-stream) stream)
+(defmethod print-object ((x echo-stream) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (format stream
             ":INPUT-STREAM ~S :OUTPUT-STREAM ~S"
@@ -1982,17 +2006,9 @@ benefit of the function GET-OUTPUT-STREAM-STRING."
 
 ;;;; Shared {READ,WRITE}-SEQUENCE support functions
 
-(declaim (inline stream-element-mode
-                 stream-compute-io-function
+(declaim (inline stream-compute-io-function
                  compatible-vector-and-stream-element-types-p))
 
-(defun stream-element-mode (stream)
-  (declare (type stream stream))
-  (if (fd-stream-p stream)
-      (fd-stream-element-mode stream)
-      (stream-element-type-stream-element-mode
-       (stream-element-type stream))))
-
 (defun stream-compute-io-function (stream
                                    stream-element-mode sequence-element-type
                                    character-io binary-io bivalent-io)
