@@ -35,7 +35,7 @@
 ;;; constructors only.
 #!+(or sb-eval sb-fasteval)
 (defun %make-structure-instance (dd slot-specs &rest slot-values)
-  (let ((instance (%make-instance (dd-instance-length dd))))
+  (let ((instance (%make-instance (dd-length dd)))) ; length = sans header word
     (setf (%instance-layout instance) (dd-layout-or-lose dd))
     (mapc (lambda (spec value)
             (destructuring-bind (raw-type . index) (cdr spec)
@@ -54,46 +54,6 @@
           slot-specs slot-values)
     instance))
 
-(defun %raw-instance-ref/word (instance index)
-  (declare (type index index))
-  (%raw-instance-ref/word instance index))
-(defun %raw-instance-set/word (instance index new-value)
-  (declare (type index index)
-           (type sb!vm:word new-value))
-  (%raw-instance-set/word instance index new-value))
-
-(defun %raw-instance-ref/single (instance index)
-  (declare (type index index))
-  (%raw-instance-ref/single instance index))
-(defun %raw-instance-set/single (instance index new-value)
-  (declare (type index index)
-           (type single-float new-value))
-  (%raw-instance-set/single instance index new-value))
-
-(defun %raw-instance-ref/double (instance index)
-  (declare (type index index))
-  (%raw-instance-ref/double instance index))
-(defun %raw-instance-set/double (instance index new-value)
-  (declare (type index index)
-           (type double-float new-value))
-  (%raw-instance-set/double instance index new-value))
-
-(defun %raw-instance-ref/complex-single (instance index)
-  (declare (type index index))
-  (%raw-instance-ref/complex-single instance index))
-(defun %raw-instance-set/complex-single (instance index new-value)
-  (declare (type index index)
-           (type (complex single-float) new-value))
-  (%raw-instance-set/complex-single instance index new-value))
-
-(defun %raw-instance-ref/complex-double (instance index)
-  (declare (type index index))
-  (%raw-instance-ref/complex-double instance index))
-(defun %raw-instance-set/complex-double (instance index new-value)
-  (declare (type index index)
-           (type (complex double-float) new-value))
-  (%raw-instance-set/complex-double instance index new-value))
-
 (defun %instance-layout (instance)
   (%instance-layout instance))
 
@@ -139,18 +99,14 @@
          (layout (classoid-layout classoid)))
     (when (eq (dd-pure dd) t)
       (setf (layout-pure layout) t))
-    #!+interleaved-raw-slots
     ;; Make a vector of EQUALP slots comparators, indexed by (- word-index data-start).
     ;; This has to be assigned to something regardless of whether there are
     ;; raw slots just in case someone mutates a layout which had raw
     ;; slots into one which does not - although that would probably crash
     ;; unless no instances exist or all raw slots miraculously contained
     ;; bits which were the equivalent of valid Lisp descriptors.
-    ;;
-    ;; It's not worth adding a #-interleaved-raw-slots case to this optimization
-    ;; because every architecture can be made to use the new approach.
     (setf (layout-equalp-tests layout)
-          (if (zerop (layout-untagged-bitmap layout))
+          (if (zerop (layout-bitmap layout))
               #()
               ;; The initial element of NIL means "do not compare".
               ;; Ignored words (comparator = NIL) fall into two categories:
@@ -187,28 +143,20 @@
   (let ((layout (%instance-layout structure)))
     (when (layout-invalid layout)
       (error "attempt to copy an obsolete structure:~%  ~S" structure))
-    (let ((res (%make-instance (%instance-length structure)))
-          (len (layout-length layout)))
+    ;; Previously this had to used LAYOUT-LENGTH in the allocation,
+    ;; to avoid copying random bits from the stack to the heap if you had a
+    ;; padding word in a stack-allocated instance. This is no longer an issue.
+    ;; %INSTANCE-LENGTH returns the number of words that are logically in the
+    ;; instance, with no padding. Using %INSTANCE-LENGTH allows potentially
+    ;; interesting nonstandard things like variable-length structures.
+    (let* ((len (%instance-length structure))
+           (res (%make-instance len)))
       (declare (type index len))
-      #!-interleaved-raw-slots
-      (let ((nuntagged (layout-n-untagged-slots layout)))
-        ;; Copy ordinary slots including the layout.
-        (dotimes (i (- len nuntagged))
-          (declare (type index i))
-          (setf (%instance-ref res i) (%instance-ref structure i)))
-        ;; Copy raw slots.
-        (dotimes (i nuntagged)
-          (declare (type index i))
-          (setf (%raw-instance-ref/word res i)
-                (%raw-instance-ref/word structure i))))
-      #!+interleaved-raw-slots
-      (let ((metadata (layout-untagged-bitmap layout)))
+      (let ((bitmap (layout-bitmap layout)))
         ;; Don't assume that %INSTANCE-REF can access the layout.
         (setf (%instance-layout res) (%instance-layout structure))
-        ;; With interleaved slots, the only difference between %instance-ref
-        ;; and %raw-instance-ref/word is the storage class of the VOP operands.
-        ;; Since x86(-64) doesn't partition the register set, the bitmap test
-        ;; could be skipped if we wanted to copy everything as raw.
+        ;; On backends which don't segregate descriptor vs. non-descriptor
+        ;; registers, we could speed up this code in an obvious way.
         (macrolet ((copy-loop (raw-p &optional step)
                      `(do ((i sb!vm:instance-data-start (1+ i)))
                           ((>= i len))
@@ -219,33 +167,16 @@
                             (setf (%instance-ref res i)
                                   (%instance-ref structure i)))
                         ,step)))
-          (cond ((zerop metadata) ; no untagged slots.
+          (cond ((zerop bitmap) ; no untagged slots.
                  (copy-loop nil))
                 ;; The fixnum case uses fixnum operations for ODDP and ASH.
-                ((fixnump metadata) ; shift and mask is faster than logbitp
-                 (copy-loop (oddp (truly-the fixnum metadata))
-                            (setq metadata (ash metadata -1))))
+                ((fixnump bitmap) ; shift and mask is faster than logbitp
+                 (copy-loop (oddp (truly-the fixnum bitmap))
+                            (setq bitmap (ash bitmap -1))))
                 (t ; bignum - use LOGBITP to avoid consing more bignums
-                 (copy-loop (logbitp i metadata))))))
+                 (copy-loop (logbitp i bitmap))))))
       res)))
 
-
-;; Do an EQUALP comparison on the raw slots (only, not the normal slots) of a
-;; structure.
-#!-interleaved-raw-slots
-(defun raw-instance-slots-equalp (layout x y)
-  ;; This implementation sucks, but hopefully EQUALP on raw structures
-  ;; won't be a major bottleneck for anyone. It'd be tempting to do
-  ;; all this with %RAW-INSTANCE-REF/WORD and bitwise comparisons, but
-  ;; that'll fail in some cases. For example -0.0 and 0.0 are EQUALP
-  ;; but have different bit patterns. -- JES, 2007-08-21
-  (loop for dsd in (dd-slots (layout-info layout))
-        for raw-type-index = (dsd-%raw-type dsd)
-        always (or (eql raw-type-index -1)
-                   (funcall (raw-slot-data-comparer
-                             (svref *raw-slot-data* raw-type-index))
-                            (dsd-index dsd) x y))))
-
 ;;; default PRINT-OBJECT method
 
 ;;; Printing formerly called the auto-generated accessor functions,
@@ -330,60 +261,7 @@
                             #'%default-structure-ugly-print)
                         structure stream name dd))))))
 
-(def!method print-object ((x structure-object) stream)
+(defmethod print-object ((x structure-object) stream)
   (default-structure-print x stream *current-level-in-print*))
-
-;; This generates a sexpr that can be recognized as having a particular
-;; shape so that the dumping mechanism can decide if it is or is not
-;; necessary to run that code through the main compiler - FASL operations
-;; can be used in the case that all slots are preserved.
-;; In particular, there are no gensyms in the result, so that calling this
-;; twice on the same object yields the same list as compared by EQUAL.
-(defun structure-obj-slot-saving-forms (struct slot-names slot-names-p)
-  (let* ((layout (%instance-layout struct))
-         (dd (layout-info layout)))
-    (mapcan (lambda (dsd)
-              (when (or (not slot-names-p) (memq (dsd-name dsd) slot-names))
-                (let ((index (dsd-index dsd))
-                      (rsd (dsd-raw-slot-data dsd)))
-                  (if (not rsd)
-                      `((%instance-ref ,struct ,index)
-                        ,(let ((val (%instance-ref struct index)))
-                           (if (and (or (listp val) (symbolp val))
-                                    (not (member val '(nil t))))
-                               (list 'quote val)
-                               val)))
-                      (let ((accessor (raw-slot-data-accessor-name rsd)))
-                        `((,accessor ,struct ,index)
-                          ,(funcall accessor struct index)))))))
-            (dd-slots dd))))
-
-;; Return T if CREATION-FORM and INIT-FORM would have the identical effect
-;; as :SB-JUST-DUMP-IT-NORMALLY for STRUCT. MAKE-LOAD-FORM-SAVING-SLOTS can't
-;; merely return the magic token (when possible) because a user application
-;; could call MAKE-LOAD-FORM-SAVING-SLOTS to obtain forms that can be evaluated
-;; or otherwise examined. So instead we scan the code and detect whether it is
-;; identical to what was returned from a trivial use of M-L-F-S-S.
-(defun canonical-slot-saving-forms-p (struct creation-form init-form)
-  (and (sb!c::canonical-instance-maker-form-p creation-form)
-       (typep init-form '(cons (eql setf)))
-       (eq (cadr (cadr (cadr creation-form))) (class-name (class-of struct)))
-       (= (length (dd-slots (layout-info (%instance-layout struct))))
-          (ash (list-length (cdr init-form)) -1))
-       (flet ((eq-quoted-p (a b)
-                (and (typep a '(cons (eql quote) (cons t null)))
-                     (typep b '(cons (eql quote) (cons t null)))
-                     (eq (cadr a) (cadr b)))))
-         ;; Naively, EQUALP would almost work to compare the slot assignments,
-         ;; but we must not get stuck in circular lists, so traverse by hand.
-         (loop for (expect-place expect-value)
-               on (structure-obj-slot-saving-forms struct nil nil) by #'cddr
-               for (actual-place actual-value) on (cdr init-form) by #'cddr
-               always (and (equal actual-place expect-place)
-                           ;; Use EQL, not EQ. Values come from the identical
-                           ;; struct, but reading a raw slot can create new
-                           ;; pointers. For QUOTE forms, EQ is ok.
-                           (or (eql actual-value expect-value)
-                               (eq-quoted-p actual-value expect-value)))))))
 
 (/show0 "target-defstruct.lisp end of file")
