@@ -222,7 +222,6 @@
 
 (def!struct (ctype (:conc-name type-)
                    (:constructor nil)
-                   (:make-load-form-fun make-type-load-form)
                    #-sb-xc-host (:pure t))
   ;; the class of this type
   ;;
@@ -362,7 +361,10 @@
                                 (,(!type-class-fun-slot name) parent))))))))
     #-sb-xc
     `(if (find ',name *type-classes* :key #'type-class-name)
-         (warn "Not redefining type-class ~S" ',name)
+         ;; Careful: type-classes are very complicated things to redefine.
+         ;; For the sake of parallelized make-host-1 we have to allow it
+         ;; not to be an error to get here, but we can't overwrite anything.
+         (style-warn "Not redefining type-class ~S" ',name)
          (vector-push-extend ,make-it *type-classes*))
     ;; The Nth entry in the array of classes contain a list of instances
     ;; of the type-class created by genesis that need patching.
@@ -688,7 +690,6 @@
 
 (in-package "SB!ALIEN")
 (def!struct (alien-type
-             (:make-load-form-fun sb!kernel:just-dump-it-normally)
              (:constructor make-alien-type
                            (&key class bits alignment
                             &aux (alignment
@@ -696,6 +697,7 @@
   (class 'root :type symbol :read-only t)
   (bits nil :type (or null unsigned-byte))
   (alignment nil :type (or null unsigned-byte)))
+(!set-load-form-method alien-type (:xc :target))
 
 (in-package "SB!KERNEL")
 (defstruct (alien-type-type
