@@ -931,10 +931,9 @@ and no value was provided for it." name))))))))))
 ;;; false). If there was a problem, we return NIL.
 (defun assert-definition-type
     (functional type &key (really-assert t)
-     ((:lossage-fun *lossage-fun*)
-      #'compiler-style-warn)
-     unwinnage-fun
-     (where "previous declaration"))
+                          ((:lossage-fun *lossage-fun*) #'compiler-style-warn)
+                          unwinnage-fun
+                          (where "previous declaration"))
   (declare (type functional functional)
            (type function *lossage-fun*)
            (string where))
@@ -957,15 +956,22 @@ and no value was provided for it." name))))))))))
           ((and dtype (not (values-types-equal-or-intersect dtype
                                                             type-returns)))
            (note-lossage
-            "The result type from ~A:~%  ~S~@
-             conflicts with the definition's result type:~%  ~S"
-            where (type-specifier type-returns) (type-specifier dtype))
+             "The result type from ~A:~%  ~
+              ~/sb!impl:print-type/~@
+              conflicts with the definition's result type:~%  ~
+              ~/sb!impl:print-type/"
+            where type-returns dtype)
            nil)
           (*lossage-detected* nil)
           ((not really-assert) t)
           (t
+           ;; REALLY-ASSERT can be T or `(:NOT . ,vars) where the latter is
+           ;; a list of vars for which compiling will *not* generate
+           ;; an automatic check.
            (let ((policy (lexenv-policy (functional-lexenv functional))))
-             (when (policy policy (> type-check 0))
+             (when (and (policy policy (> type-check 0))
+                        (or (eq really-assert t)
+                            (not (member :result (cdr really-assert)))))
                (assert-lvar-type (return-result return) type-returns
                                  policy)))
            (loop for var in vars and type in types do
@@ -973,12 +979,15 @@ and no value was provided for it." name))))))))))
                         (when (and unwinnage-fun
                                    (not (csubtypep (leaf-type var) type)))
                           (funcall unwinnage-fun
-                                   "Assignment to argument: ~S~%  ~
-                                    prevents use of assertion from function ~
-                                    type ~A:~%  ~S~%"
-                                   (leaf-debug-name var)
-                                   where
-                                   (type-specifier type))))
+                                   #.(#+sb-xc sb!impl::!xc-preprocess-format-control
+                                      #-sb-xc identity
+                                    "Assignment to argument: ~S~%  ~
+                                     prevents use of assertion from function ~
+                                     type ~A:~% ~/sb!impl:print-type/~%")
+                                   (leaf-debug-name var) where type)))
+                       ((and (listp really-assert) ; (:NOT . ,vars)
+                             (member (lambda-var-%source-name var)
+                                     (cdr really-assert)))) ; do nothing
                        (t
                         (setf (leaf-type var) type)
                         (let ((s-type (make-single-value-type type)))
@@ -986,6 +995,15 @@ and no value was provided for it." name))))))))))
                             (derive-node-type ref s-type))))))
            t))))))
 
+;;; Manipulate the poorly-named :REALLY-ASSERT value.
+;;; It would make sense to pass the opposite sense of the arg
+;;; (as ":SKIP-CHECKS") corresponding to the declaration.
+(defun explicit-check->really-assert (explicit-check)
+  (case explicit-check
+    ((nil) t)
+    ((t) nil)
+    (t `(:not . ,explicit-check))))
+
 ;;; FIXME: This is quite similar to ASSERT-NEW-DEFINITION.
 (defun assert-global-function-definition-type (name fun)
   (declare (type functional fun))
@@ -999,7 +1017,7 @@ and no value was provided for it." name))))))))))
            fun type
            :unwinnage-fun #'compiler-notify
            :where "proclamation"
-           :really-assert (not explicit-check)))
+           :really-assert (explicit-check->really-assert explicit-check)))
         ;; Can't actually test this. DEFSTRUCTs declare this, but non-toplevel
         ;; ones won't have an FTYPE at compile-time.
         #+nil
@@ -1095,17 +1113,15 @@ and no value was provided for it." name))))))))))
       (let ((sources (lvar-all-sources tag)))
         (if (singleton-p sources)
             (compiler-style-warn
-             "~@<using ~S of type ~S as a catch tag (which ~
-                 tends to be unportable because THROW and CATCH ~
-                 use EQ comparison)~@:>"
-             (first sources)
-             (type-specifier (lvar-type tag)))
+              "~@<Using ~S of type ~/sb!impl:print-type/ as ~
+               a catch tag (which tends to be unportable because THROW ~
+               and CATCH use EQ comparison)~@:>"
+             (first sources) (lvar-type tag))
             (compiler-style-warn
-             "~@<using ~{~S~^~#[~; or ~:;, ~]~} in ~S of type ~S ~
-                 as a catch tag (which tends to be unportable ~
-                 because THROW and CATCH use EQ comparison)~@:>"
-             (rest sources) (first sources)
-             (type-specifier (lvar-type tag))))))))
+              "~@<Using ~{~S~^~#[~; or ~:;, ~]~} in ~S of type ~
+               ~/sb!impl:print-type/ as a catch tag (which tends to be ~
+               unportable because THROW and CATCH use EQ comparison)~@:>"
+             (rest sources) (first sources) (lvar-type tag)))))))
 
 (defun %compile-time-type-error (values atype dtype detail context)
   (declare (ignore dtype))
@@ -1115,8 +1131,9 @@ and no value was provided for it." name))))))))))
                  :datum (car values)
                  :expected-type atype
                  :format-control
-                 "~@<Value set ~2I~_[~{~S~^ ~}] ~I~_from ~S in~_~A ~I~_is ~
-                   not of type ~2I~_~S.~:>"
+                  "~@<Value set ~2I~_[~{~S~^ ~}] ~I~_from ~S in~_~A ~
+                   ~I~_is not of type ~
+                   ~2I~_~/sb!impl:print-type-specifier/.~:>"
                  :format-arguments (list values
                                          (first detail) context
                                          atype))
@@ -1124,9 +1141,10 @@ and no value was provided for it." name))))))))))
                  :datum (car values)
                  :expected-type atype
                  :format-control
-                 "~@<Value set ~2I~_[~{~S~^ ~}] ~
+                  "~@<Value set ~2I~_[~{~S~^ ~}] ~
                    ~I~_from ~2I~_~{~S~^~#[~; or ~:;, ~]~} ~
-                   ~I~_of ~2I~_~S ~I~_in~_~A ~I~_is not of type ~2I~_~S.~:>"
+                   ~I~_of ~2I~_~S ~I~_in~_~A ~I~_is not of type ~
+                   ~2I~_~/sb!impl:print-type-specifier/.~:>"
                  :format-arguments (list values
                                          (rest detail) (first detail)
                                          context
@@ -1135,17 +1153,19 @@ and no value was provided for it." name))))))))))
           (error 'simple-type-error
                  :datum (car values)
                  :expected-type atype
-                 :format-control "~@<Value of ~S in ~_~A ~I~_is ~2I~_~S, ~
-                                ~I~_not a ~2I~_~S.~:@>"
+                 :format-control
+                  "~@<Value of ~S in ~_~A ~I~_is ~2I~_~S, ~
+                   ~I~_not a ~2I~_~/sb!impl:print-type-specifier/.~:@>"
                  :format-arguments (list (car detail) context
                                          (car values)
                                          atype))
           (error 'simple-type-error
                  :datum (car values)
                  :expected-type atype
-                 :format-control "~@<Value from ~2I~_~{~S~^~#[~; or ~:;, ~]~} ~
-                                   ~I~_of ~2I~_~S ~I~_in~_~A ~I~_is ~2I~_~S, ~
-                                   ~I~_not a ~2I~_~S.~:@>"
+                 :format-control
+                  "~@<Value from ~2I~_~{~S~^~#[~; or ~:;, ~]~} ~
+                   ~I~_of ~2I~_~S ~I~_in~_~A ~I~_is ~2I~_~S, ~
+                   ~I~_not a ~2I~_~/sb!impl:print-type-specifier/.~:@>"
                  :format-arguments (list (rest detail) (first detail) context
                                          (car values)
                                          atype)))))
@@ -1165,19 +1185,23 @@ and no value was provided for it." name))))))))))
                 (if (constantp detail)
                     (warn 'type-warning
                           :format-control
-                          "~@<Constant ~2I~_~S ~Iconflicts with its ~
-                              asserted type ~2I~_~S.~@:>"
+                           "~@<Constant ~2I~_~S ~Iconflicts with its ~
+                            asserted type ~
+                            ~2I~_~/sb!impl::print-type-specifier/.~@:>"
                           :format-arguments (list (eval detail) atype))
                     (warn 'type-warning
                           :format-control
-                          "~@<Derived type of ~S is ~2I~_~S, ~
-                              ~I~_conflicting with ~
-                              its asserted type ~2I~_~S.~@:>"
+                           "~@<Derived type of ~S is ~2I~_~S, ~
+                            ~I~_conflicting with its asserted type ~
+                            ~2I~_~/sb!impl:print-type-specifier/.~@:>"
                           :format-arguments (list detail dtype atype))))
               (warn 'type-warning
                     :format-control
-                    "~@<Derived type of ~2I~_~{~S~^~#[~; and ~:;, ~]~} ~
-                     ~I~_in ~2I~_~S ~I~_is ~2I~_~S, ~I~_conflicting with ~
-                     their asserted type ~2I~_~S.~@:>"
-                    :format-arguments (list (rest detail) (first detail) dtype atype)))))
+                     "~@<Derived type of ~2I~_~{~S~^~#[~; and ~:;, ~
+                      ~]~} ~I~_in ~2I~_~S ~I~_is ~
+                      ~2I~_~/sb!impl:print-type-specifier/, ~
+                      ~I~_conflicting with their asserted type ~
+                      ~2I~_~/sb!impl:print-type-specifier/.~@:>"
+                    :format-arguments (list (rest detail) (first detail)
+                                            dtype atype)))))
     (ir2-convert-full-call node block)))
