@@ -13,7 +13,7 @@
 
 ;;; types and defaults
 
-(def!constant label-column-width 7)
+(defconstant label-column-width 7)
 
 (deftype text-width () '(integer 0 1000))
 (deftype alignment () '(integer 0 64))
@@ -22,7 +22,7 @@
 (deftype disassem-length () '(unsigned-byte 24))
 (deftype column () '(integer 0 1000))
 
-(def!constant max-filtered-value-index 32)
+(defconstant max-filtered-value-index 32)
 (deftype filtered-value-index ()
   `(integer 0 (,max-filtered-value-index)))
 (deftype filtered-value-vector ()
@@ -83,15 +83,15 @@
                  dchunk=
                  dchunk-count-bits))
 
-(def!constant dchunk-bits #.sb!vm:n-word-bits)
+(defconstant dchunk-bits sb!vm:n-word-bits)
 
 (deftype dchunk ()
   `(unsigned-byte ,dchunk-bits))
 (deftype dchunk-index ()
   `(integer 0 ,dchunk-bits))
 
-(def!constant dchunk-zero 0)
-(def!constant dchunk-one #.(1- (expt 2 sb!vm:n-word-bits)))
+(defconstant dchunk-zero 0)
+(defconstant dchunk-one (1- (expt 2 sb!vm:n-word-bits)))
 
 (defun dchunk-extract (chunk byte-spec)
   (declare (type dchunk chunk))
@@ -190,13 +190,9 @@
 
 (defstruct (instruction (:conc-name inst-)
                         (:constructor
-                         make-instruction (name
-                                           format-name
-                                           print-name
-                                           length
-                                           mask id
-                                           printer
-                                           labeller prefilters control))
+                         make-instruction (name format-name print-name
+                                           length mask id printer labeller
+                                           prefilters control))
                         (:copier nil))
   (name nil :type (or symbol string) :read-only t)
   (format-name nil :type (or symbol string) :read-only t)
@@ -209,15 +205,15 @@
   (print-name nil :type symbol :read-only t)
 
   ;; disassembly "functions"
-  (prefilters nil :type list)
-  (labeller nil :type (or list vector))
-  (printer (missing-arg) :type (or null function))
+  (prefilters nil :type list :read-only t)
+  (labeller nil :type (or list vector) :read-only t)
+  (printer nil :type (or null function) :read-only t)
   (control nil :type (or null function) :read-only t)
 
   ;; instructions that are the same as this instruction but with more
   ;; constraints
   (specializers nil :type list))
-(def!method print-object ((inst instruction) stream)
+(defmethod print-object ((inst instruction) stream)
   (print-unreadable-object (inst stream :type t :identity t)
     (format stream "~A(~A)" (inst-name inst) (inst-format-name inst))))
 
@@ -228,7 +224,7 @@
                        (:copier nil))
   (valid-mask dchunk-zero :type dchunk) ; applies to *children*
   (choices nil :type list))
-(def!method print-object ((ispace inst-space) stream)
+(defmethod print-object ((ispace inst-space) stream)
   (print-unreadable-object (ispace stream :type t :identity t)))
 
 ;;; now that we've defined the structure, we can declaim the type of
@@ -239,70 +235,10 @@
                               (:copier nil))
   (common-id dchunk-zero :type dchunk)  ; applies to *parent's* mask
   (subspace (missing-arg) :type (or inst-space instruction)))
-
-(defmacro !begin-instruction-definitions () nil) ; FIXME: remove
 
-;;; FIXME: If we we interned the temp vars,
-;;; and wouldn't use symbols qua strings, then this would reduce to EQUAL.
-(defun equal-mod-gensyms (a b)
-  (named-let recurse ((a a) (b b))
-    (etypecase a
-      (null (null b))
-      (list (and (listp b) (recurse (car a) (car b)) (recurse (cdr a) (cdr b))))
-      (symbol (or (eq a b)
-                  (and (symbolp b)
-                       (not (symbol-package a))
-                       (not (symbol-package b))
-                       ;; If "strings", then comparison by STRING= is right,
-                       ;; and if lexical vars, it's also right because
-                       ;; we never rebind a given temp within a function.
-                       (string= a b))))
-      ((or number character function) (eql a b))
-      (vector (and (vectorp b) (every #'recurse a b))))))
-
-;;; Previously there were complicated checker functions which tried to attempt to
-;;; decide, given two FUNSTATEs, whether all their args were similarly used,
-;;; where "similarity" required that the prefilter and such be identical.
-;;; Instead we can just look at two sexprs and decide whether they act the same,
-;;; which is of course impossible in general; however, for this purpose,
-;;; if sexprs are EQUAL disregarding variations in gensyms, then their code
-;;; can be folded. If we miss (don't fold) things that act the same, it's ok.
-;;; N.B.: This definition of equivalence is admissible because there can be
-;;; no "interesting" non-null lexical environment. While it could be non-null,
-;;; it can't matter, because our auto-generated code can't depend on the lexenv.
-(defvar *current-instruction-flavor* nil)
-(defun generate-function (kind forms funstate code-folding-cache skeleton)
-  (let* ((sub-table (assq kind code-folding-cache))
-         (bindings (make-arg-temp-bindings funstate))
-         (guts `(let* ,bindings ,@forms)))
-    (or (cdr (assoc guts (cdr sub-table) :test #'equal-mod-gensyms))
-        (let* ((name (concatenate 'string "INST-" (string kind) "-"
-                                  (write-to-string (length sub-table))))
-               (definition
-                 (compile nil
-                   `(named-lambda ,name ,@(subst guts :body (cdr skeleton))))))
-          (push (cons guts definition) (cdr sub-table))
-          definition))))
-
-(defstruct (arg (:copier nil)
-                (:predicate nil)
-                (:constructor %make-arg (name))
-                (:constructor standard-make-arg) ; only so #S readmacro works
-                (:print-object
-                 (lambda (self stream)
-                   (if *print-readably*
-                       (call-next-method)
-                       (print-unreadable-object (self stream :type t)
-                         (format stream
-                                 "~A ~:[~;+~]~:S~@[=~S~]~@[ filt=~S~]~
-~@[ lbl=~S~]~@[ prt=~S~]"
-                                 (arg-name self)
-                                 (arg-sign-extend-p self)
-                                 (arg-fields self)
-                                 (arg-value self)
-                                 (arg-prefilter self)
-                                 (arg-use-label self)
-                                 (arg-printer self)))))))
+(defstruct (arg (:constructor %make-arg (name))
+                (:copier nil)
+                (:predicate nil))
   (name nil :type symbol)
   (fields nil :type list)
 
@@ -340,17 +276,11 @@
   (or (car (assoc name funstate :key #'arg-name :test #'eq))
       (pd-error "unknown argument ~S" name)))
 
-;;;; Since we can't include some values in compiled output as they are
-;;;; (notably functions), we sometimes use a VALSRC structure to keep
-;;;; track of the source from which they were derived.
-
 ;;; machinery to provide more meaningful error messages during compilation
+(defvar *current-instruction-flavor*)
 (defun pd-error (fmt &rest args)
-  (if *current-instruction-flavor*
-      (error "~@<in printer-definition for ~S(~S): ~3I~:_~?~:>"
-             (car *current-instruction-flavor*)
-             (cdr *current-instruction-flavor*)
-             fmt args)
+  (if (boundp '*current-instruction-flavor*)
+      (error "~{A printer ~D~}: ~?" *current-instruction-flavor* fmt args)
       (apply #'error fmt args)))
 
 (defun format-or-lose (name)
@@ -479,6 +409,7 @@
                                 (arg (find ',(car arg-spec) format-args
                                            :key #'arg-name))
                                 (funstate (make-funstate format-args))
+                                (*!temp-var-counter* 0)
                                 (expr (arg-value-form arg funstate :numeric)))
                            `(let* ,(make-arg-temp-bindings funstate) ,expr))))
                     (reader)))))))
@@ -585,24 +516,30 @@
 ;;; RENDERING style, which is one of :RAW, :SIGN-EXTENDED,
 ;;; :FILTERED, :NUMERIC, and :FINAL. Each rendering depends on the preceding
 ;;; one, so asking for :FINAL will implicitly compute all renderings.
+(defvar *!temp-var-counter*)
 (defun gen-arg-forms (arg rendering funstate)
-  (let* ((arg-cell (assq arg funstate))
-         (rendering-temps (cdr (assq rendering (cdr arg-cell))))
-         (vars (car rendering-temps))
-         (forms (cdr rendering-temps)))
-    (unless forms
-      (multiple-value-bind (new-forms single-value-p)
-          (%gen-arg-forms arg rendering funstate)
-        (setq forms new-forms
-              vars (cond ((or single-value-p (atom forms))
-                          (if (symbolp forms) vars (sb!xc:gensym "_")))
-                         ((every #'symbolp forms)
-                          ;; just use the same as the forms
-                          nil)
-                         (t
-                          (make-gensym-list (length forms) "_"))))
-        (push (list* rendering vars forms) (cdr arg-cell))))
-    (or vars forms)))
+  (labels ((tempvars (n)
+             (if (plusp n)
+                 (cons (intern (format nil ".T~D" (incf *!temp-var-counter*))
+                               (load-time-value (find-package "SB!DISASSEM")))
+                       (tempvars (1- n))))))
+    (let* ((arg-cell (assq arg funstate))
+           (rendering-temps (cdr (assq rendering (cdr arg-cell))))
+           (vars (car rendering-temps))
+           (forms (cdr rendering-temps)))
+      (unless forms
+        (multiple-value-bind (new-forms single-value-p)
+            (%gen-arg-forms arg rendering funstate)
+          (setq forms new-forms
+                vars (cond ((or single-value-p (atom forms))
+                            (if (symbolp forms) vars (car (tempvars 1))))
+                           ((every #'symbolp forms)
+                            ;; just use the same as the forms
+                            nil)
+                           (t
+                            (tempvars (length forms)))))
+          (push (list* rendering vars forms) (cdr arg-cell))))
+      (or vars forms))))
 
 (defun maybe-listify (forms)
   (cond ((atom forms)
@@ -695,19 +632,21 @@
            `((lookup-label ,(maybe-listify numeric-forms)))
            numeric-forms)))))
 
-(defun find-printer-fun (printer-source args cache)
-  (let ((source (preprocess-printer printer-source args))
-        (funstate (make-funstate args)))
-   (generate-function
-    :printer
-    (let ((sb!xc:*gensym-counter* 0)) (compile-printer-list source funstate))
-    funstate
-    cache
-    '(lambda (chunk inst stream dstate)
-       (declare (type dchunk chunk)
-                (type instruction inst)
-                (type stream stream)
-                (type disassem-state dstate))
+(defun find-printer-fun (printer-source args cache *current-instruction-flavor*)
+  (let* ((source (preprocess-printer printer-source args))
+         (funstate (make-funstate args))
+         (forms (let ((*!temp-var-counter* 0))
+                  (compile-printer-list source funstate)))
+         (bindings (make-arg-temp-bindings funstate))
+         (guts `(let* ,bindings ,@forms))
+         (sub-table (assq :printer cache)))
+    (or (cdr (assoc guts (cdr sub-table) :test #'equal))
+        (let ((template
+     '(lambda (chunk inst stream dstate
+               &aux (chunk (truly-the dchunk chunk))
+                    (inst (truly-the instruction inst))
+                    (stream (truly-the stream stream))
+                    (dstate (truly-the disassem-state dstate)))
        (macrolet ((local-format-arg (arg fmt)
                     `(funcall (formatter ,fmt) stream ,arg)))
          (flet ((local-tab-to-arg-column ()
@@ -748,8 +687,10 @@
                             local-call-arg-printer local-call-global-printer
                             local-filtered-value local-extract
                             lookup-label adjust-label))
-           :body))))))
-
+           :body)))))
+          (cdar (push (cons guts (compile nil (subst guts :body template)))
+                      (cdr sub-table)))))))
+
 (defun preprocess-test (subj form args)
   (multiple-value-bind (subj test)
       (if (and (consp form) (symbolp (car form)) (not (keywordp (car form))))
@@ -893,24 +834,7 @@
       (return choice))))
 
 (defun compile-printer-list (sources funstate)
-  (unless (null sources)
-    ;; Coalesce adjacent symbols/strings, and convert to strings if possible,
-    ;; since they require less consing to write.
-    (do ((el (car sources) (car sources))
-         (names nil (cons (strip-quote el) names)))
-        ((not (string-or-qsym-p el))
-         (when names
-           ;; concatenate adjacent strings and symbols
-           (let ((string
-                  (apply #'concatenate
-                         'string
-                         (mapcar #'string (nreverse names)))))
-             ;; WTF? Everything else using INST-PRINT-NAME writes a string.
-             (push (if (some #'alpha-char-p string)
-                       `',(make-symbol string) ; Preserve casifying output.
-                       string)
-                   sources))))
-      (pop sources))
+  (when sources
     (cons (compile-printer-body (car sources) funstate)
           (compile-printer-list (cdr sources) funstate))))
 
@@ -983,18 +907,6 @@
        `(,(if (arg-use-label arg) 'local-princ16 'local-princ)
          ,(arg-value-form arg funstate))))))
 
-(defun string-or-qsym-p (thing)
-  (or (stringp thing)
-      (and (consp thing)
-           (eq (car thing) 'quote)
-           (or (stringp (cadr thing))
-               (symbolp (cadr thing))))))
-
-(defun strip-quote (thing)
-  (if (and (consp thing) (eq (car thing) 'quote))
-      (cadr thing)
-      thing))
-
 (defun compare-fields-form (val-form-1 val-form-2)
   (flet ((listify-fields (fields)
            (cond ((symbolp fields) fields)
@@ -1120,13 +1032,6 @@
 (defun princ16 (value stream)
   (write value :stream stream :radix t :base 16 :escape nil))
 
-(declaim (ftype function read-suffix))
-(defun read-signed-suffix (length dstate)
-  (declare (type (member 8 16 32 64) length)
-           (type disassem-state dstate)
-           (optimize (speed 3) (safety 0)))
-  (sign-extend (read-suffix length dstate) length))
-
 (defstruct (storage-info (:copier nil))
   (groups nil :type list)               ; alist of (name . location-group)
   (debug-vars #() :type vector))
@@ -1138,10 +1043,6 @@
              :type (function () sb!sys:system-area-pointer))
   ;; Length in bytes of the range of memory covered by this segment.
   (length 0 :type disassem-length)
-  ;; Length of the memory range excluding any trailing untagged data.
-  ;; Defaults to 'length' but could be shorter.
-  ;; FIXME: can opcodes-length really be shorter? Nothing ever alters it.
-  (opcodes-length 0 :type disassem-length)
   (virtual-location 0 :type address)
   (storage-info nil :type (or null storage-info))
   ;; KLUDGE: CODE-COMPONENT is not a type the host understands
@@ -1200,7 +1101,7 @@
 
   ;; currently active source variables
   (current-valid-locations nil :type (or null (vector bit))))
-(def!method print-object ((dstate disassem-state) stream)
+(defmethod print-object ((dstate disassem-state) stream)
   (print-unreadable-object (dstate stream :type t)
     (format stream
             "+~W~@[ in ~S~]"
@@ -1233,3 +1134,10 @@
 ;;; DSTATE.
 (defun dstate-get-inst-prop (dstate name)
   (member name (dstate-inst-properties dstate) :test #'eq))
+
+(declaim (ftype function read-suffix))
+(defun read-signed-suffix (length dstate)
+  (declare (type (member 8 16 32 64) length)
+           (type disassem-state dstate)
+           (optimize (speed 3) (safety 0)))
+  (sign-extend (read-suffix length dstate) length))
