@@ -178,7 +178,7 @@
          (fop-argc
           (if (>= val +2-operand-fops+)
               2
-              (sbit (car *fop-signatures*) (ash val -2)))))
+              (sbit (car **fop-signatures**) (ash val -2)))))
     (cond
       ((not (eql (length args) fop-argc))
        (error "~S takes ~D argument~:P" fs fop-argc))
@@ -207,6 +207,12 @@
     (dump-fop 'fop-pop fasl-output)
     (incf (fasl-output-table-free fasl-output))))
 
+(defun dump-to-table (fasl-output)
+  (prog1
+      (fasl-output-table-free fasl-output)
+    (dump-fop 'fop-move-to-table fasl-output)
+    (incf (fasl-output-table-free fasl-output))))
+
 ;;; If X is in File's EQUAL-TABLE, then push the object and return T,
 ;;; otherwise NIL.
 (defun equal-check-table (x fasl-output)
@@ -231,27 +237,24 @@
 ;;; be on the top of the FOP stack.
 (defun eq-save-object (x fasl-output)
   (declare (type fasl-output fasl-output))
-  (let ((handle (dump-pop fasl-output)))
-    (setf (gethash x (fasl-output-eq-table fasl-output)) handle)
-    (dump-push handle fasl-output))
+  (setf (gethash x (fasl-output-eq-table fasl-output))
+        (dump-to-table fasl-output))
   (values))
 (defun equal-save-object (x fasl-output)
   (declare (type fasl-output fasl-output))
-  (let ((handle (dump-pop fasl-output)))
+  (let ((handle (dump-to-table fasl-output)))
     (setf (gethash x (fasl-output-equal-table fasl-output)) handle)
-    (setf (gethash x (fasl-output-eq-table fasl-output)) handle)
-    (dump-push handle fasl-output))
+    (setf (gethash x (fasl-output-eq-table fasl-output)) handle))
   (values))
 (defun string-save-object (x fasl-output)
   (declare (type fasl-output fasl-output)
            (type string x))
-  (let ((handle (dump-pop fasl-output)))
+  (let ((handle (dump-to-table fasl-output)))
     (push (cons #+sb-xc-host 'base-char ; repeatable xc fasls
                 #-sb-xc-host (array-element-type x)
                 handle)
           (gethash x (fasl-output-equal-table fasl-output)))
-    (setf (gethash x (fasl-output-eq-table fasl-output)) handle)
-    (dump-push handle fasl-output))
+    (setf (gethash x (fasl-output-eq-table fasl-output)) handle))
   (values))
 ;;; Record X in File's CIRCULARITY-TABLE. This is called on objects
 ;;; that we are about to dump might have a circular path through them.
@@ -294,6 +297,8 @@
       ;; Before the actual FASL header, write a shebang line using the current
       ;; runtime path, so our fasls can be executed directly from the shell.
       (when *runtime-pathname*
+        #+sb-xc-host (bug "Can't write shebang line") ; no #'NATIVE-PATHNAME
+        #-sb-xc-host
         (fasl-write-string
          (format nil "#!~A --script~%"
                  (native-namestring *runtime-pathname* :as-file t))
@@ -407,7 +412,7 @@
                   (float (dump-float x file))
                   (integer (dump-integer x file)))
                 (equal-save-object x file)))
-             #!+sb-simd-pack
+             #!+(and (not (host-feature sb-xc-host)) sb-simd-pack)
              (simd-pack
               (unless (equal-check-table x file)
                 (dump-fop 'fop-simd-pack file)
@@ -758,7 +763,8 @@
 (defun dump-array (x file)
   (if (vectorp x)
       (dump-vector x file)
-      (dump-multi-dim-array x file)))
+      #-sb-xc-host (dump-multi-dim-array x file)
+      #+sb-xc-host (bug "Can't dump multi-dim array")))
 
 ;;; Dump the vector object. If it's not simple, then actually dump a
 ;;; simple version of it. But we enter the original in the EQ or EQUAL
@@ -951,7 +957,8 @@
         (setq pkg sb!int:*cl-package*)))
 
     (cond ((null pkg)
-           (let ((this-base-p (typep pname 'base-string)))
+           (let ((this-base-p #+sb-xc-host t
+                              #-sb-xc-host (typep pname 'base-string)))
              (dolist (lookalike (gethash pname (fasl-output-string=-table file))
                                 (dump-fop 'fop-uninterned-symbol-save
                                           file pname-length))
@@ -960,7 +967,9 @@
                ;; preserve the type of the string (base or character) anyway,
                ;; but if we did, then this would be right also.
                ;; [what about a symbol whose name is a (simple-array nil (0))?]
-               (let ((that-base-p (typep (symbol-name lookalike) 'base-string)))
+               (let ((that-base-p
+                      #+sb-xc-host t
+                      #-sb-xc-host (typep (symbol-name lookalike) 'base-string)))
                  (when (or (and this-base-p that-base-p)
                            (and (not this-base-p) (not that-base-p)))
                    (dump-fop 'fop-copy-symbol-save file
@@ -1116,9 +1125,8 @@
       (let ((info (sb!c::debug-info-for-component component))
             (*dump-only-valid-structures* nil))
         (dump-object info fasl-output)
-        (let ((info-handle (dump-pop fasl-output)))
-          (dump-push info-handle fasl-output)
-          (push info-handle (fasl-output-debug-info fasl-output))))
+        (push (dump-to-table fasl-output)
+              (fasl-output-debug-info fasl-output)))
 
       (let ((num-consts (- header-length sb!vm:code-constants-offset)))
         (dump-fop 'fop-code fasl-output num-consts code-length))
@@ -1139,6 +1147,8 @@
                          (fasl-output-patch-table fasl-output))))
         handle))))
 
+;;; This is only called from assemfile, which doesn't exist in the target.
+#+sb-xc-host
 (defun dump-assembler-routines (code-segment length fixups routines file)
   (dump-fop 'fop-assembler-code file)
   (dump-word length file)
@@ -1146,7 +1156,9 @@
   (dolist (routine routines)
     (dump-object (car routine) file)
     (dump-fop 'fop-assembler-routine file)
-    (dump-word (label-position (cdr routine)) file))
+    (dump-word (+ (label-position (cadr routine))
+                  (caddr routine))
+               file))
   (dump-fixups fixups file)
   #!-(or x86 x86-64)
   (dump-fop 'fop-sanctify-for-execution file)
@@ -1241,6 +1253,7 @@
   (declare (type sb!c::source-info info))
   (let ((res (sb!c::debug-source-for-info info))
         (*dump-only-valid-structures* nil))
+    ;; Zero out the timestamps to get reproducible fasls.
     #+sb-xc-host (setf (sb!c::debug-source-created res) 0
                        (sb!c::debug-source-compiled res) 0)
     (dump-object res fasl-output)
@@ -1249,7 +1262,11 @@
         (dump-push res-handle fasl-output)
         (dump-fop 'fop-structset fasl-output)
         (dump-word info-handle fasl-output)
-        (dump-word sb!c::+debug-info-source-index+ fasl-output))
+        (macrolet ((debug-info-source-index ()
+                     (let ((dd (find-defstruct-description 'sb!c::debug-info)))
+                       (dsd-index (find 'source (dd-slots dd)
+                                        :key #'dsd-name :test 'string=)))))
+          (dump-word (debug-info-source-index) fasl-output)))
       #+sb-xc-host
       (progn
         (dump-push res-handle fasl-output)
@@ -1265,30 +1282,19 @@
 ;; But if it learns a layout by cross-compiling a DEFSTRUCT, that's ok too.
 (defun dump-structure (struct file)
   (when (and *dump-only-valid-structures*
-             (not (gethash struct (fasl-output-valid-structures file)))
-             #+sb-xc-host
-             (not (sb!kernel::xc-dumpable-structure-instance-p struct)))
+             (not (gethash struct (fasl-output-valid-structures file))))
     (error "attempt to dump invalid structure:~%  ~S~%How did this happen?"
            struct))
   (note-potential-circularity struct file)
   (do* ((length (%instance-length struct))
         (layout (%instance-layout struct))
-        #!-interleaved-raw-slots
-        (ntagged (- length (layout-n-untagged-slots layout)))
-        #!+interleaved-raw-slots
-        (bitmap (layout-untagged-bitmap layout))
+        (bitmap (layout-bitmap layout))
         (circ (fasl-output-circularity-table file))
-        ;; last slot first on the stack, so that the layout is on top:
-        (index (1- length) (1- index)))
-      ((< index sb!vm:instance-data-start)
+        (index sb!vm:instance-data-start (1+ index)))
+      ((>= index length)
        (dump-non-immediate-object layout file)
        (dump-fop 'fop-struct file length))
-    (let* ((obj #!-interleaved-raw-slots
-                (if (>= index ntagged)
-                    (%raw-instance-ref/word struct (- length index 1))
-                    (%instance-ref struct index))
-                #!+interleaved-raw-slots
-                (if (logbitp index bitmap)
+    (let* ((obj (if (logbitp index bitmap)
                     (%raw-instance-ref/word struct index)
                     (%instance-ref struct index)))
            (ref (gethash obj circ)))
@@ -1322,5 +1328,5 @@
   (sub-dump-object (layout-inherits obj) file)
   (sub-dump-object (layout-depthoid obj) file)
   (sub-dump-object (layout-length obj) file)
-  (sub-dump-object (layout-raw-slot-metadata obj) file)
+  (sub-dump-object (layout-bitmap obj) file)
   (dump-fop 'fop-layout file))
