@@ -64,10 +64,7 @@
 ;;; Return a list of all the nodes which use LVAR.
 (declaim (ftype (sfunction (lvar) list) find-uses))
 (defun find-uses (lvar)
-  (let ((uses (lvar-uses lvar)))
-    (if (listp uses)
-        uses
-        (list uses))))
+  (ensure-list (lvar-uses lvar)))
 
 (declaim (ftype (sfunction (lvar) lvar) principal-lvar))
 (defun principal-lvar (lvar)
@@ -443,18 +440,12 @@
     (when (eq (lambda-home fun) fun)
       (return fun))))
 
-#!-sb-fluid (declaim (inline node-block))
-(defun node-block (node)
-  (ctran-block (node-prev node)))
 (declaim (ftype (sfunction (node) component) node-component))
 (defun node-component (node)
   (block-component (node-block node)))
 (declaim (ftype (sfunction (node) physenv) node-physenv))
 (defun node-physenv (node)
   (lambda-physenv (node-home-lambda node)))
-#!-sb-fluid (declaim (inline node-dest))
-(defun node-dest (node)
-  (awhen (node-lvar node) (lvar-dest it)))
 
 #!-sb-fluid (declaim (inline node-stack-allocate-p))
 (defun node-stack-allocate-p (node)
@@ -531,7 +522,7 @@
                  (lvar-good-for-dx-p (trivial-lambda-var-ref-lvar use) dx component))))))
 
 (defun lvar-good-for-dx-p (lvar dx &optional component)
-  (let ((uses (lvar-uses lvar)))
+  (let ((uses (lvar-uses lvar))) ; TODO use ENSURE-LIST? or is it too slow?
     (if (listp uses)
         (when uses
           (every (lambda (use)
@@ -662,7 +653,7 @@
                     (handle-nested-dynamic-extent-lvars
                      dx other recheck-component)))))))
       (cons (cons dx lvar)
-            (if (listp uses)
+            (if (listp uses) ; TODO use ENSURE-LIST? or is it too slow?
                 (loop for use in uses
                       when (use-good-for-dx-p use dx recheck-component)
                       nconc (recurse use))
@@ -828,15 +819,6 @@
                                       (node-source-path use)))))
         (list (node-source-form use)))))
 
-;;; Return the unique node, delivering a value to LVAR.
-#!-sb-fluid (declaim (inline lvar-use))
-(defun lvar-use (lvar)
-  (the (not list) (lvar-uses lvar)))
-
-#!-sb-fluid (declaim (inline lvar-has-single-use-p))
-(defun lvar-has-single-use-p (lvar)
-  (typep (lvar-uses lvar) '(not list)))
-
 ;;; Return the LAMBDA that is CTRAN's home, or NIL if there is none.
 (declaim (ftype (sfunction (ctran) (or clambda null))
                 ctran-home-lambda-or-null))
@@ -868,20 +850,19 @@
 
 #!-sb-fluid (declaim (inline lvar-single-value-p))
 (defun lvar-single-value-p (lvar)
-  (or (not lvar)
-      (let ((dest (lvar-dest lvar)))
-        (typecase dest
-          ((or creturn exit)
-           nil)
-          (mv-combination
-           (eq (basic-combination-fun dest) lvar))
-          (cast
-           (locally
-               (declare (notinline lvar-single-value-p))
-             (and (cast-single-value-p dest)
-                  (lvar-single-value-p (node-lvar dest)))))
-          (t
-           t)))))
+  (or (not lvar) (%lvar-single-value-p lvar)))
+(defun %lvar-single-value-p (lvar)
+  (let ((dest (lvar-dest lvar)))
+    (typecase dest
+      ((or creturn exit)
+       nil)
+      (mv-combination
+       (eq (basic-combination-fun dest) lvar))
+      (cast
+       (and (cast-single-value-p dest)
+            (acond ((node-lvar dest) (%lvar-single-value-p it))
+                   (t t))))
+      (t t))))
 
 (defun principal-lvar-end (lvar)
   (loop for prev = lvar then (node-lvar dest)
@@ -925,7 +906,8 @@
      lambda
      cleanup handled-conditions disabled-package-locks
      policy
-     user-data)))
+     user-data
+     default)))
 
 ;;; Makes a LEXENV, suitable for using in a MACROLET introduced
 ;;; macroexpander
@@ -960,7 +942,8 @@
      (lexenv-handled-conditions lexenv)
      (lexenv-disabled-package-locks lexenv)
      (lexenv-policy lexenv)
-     (lexenv-user-data lexenv))))
+     (lexenv-user-data lexenv)
+     lexenv)))
 
 ;;;; flow/DFO/component hackery
 
@@ -1879,17 +1862,16 @@ is :ANY, the function name is not checked."
   (declare (type lvar lvar)
            (type (or symbol list) fun)
            (type index num-args))
-  (let ((fun (if (listp fun) fun (list fun))))
-    (let ((inside (lvar-uses lvar)))
-      (unless (combination-p inside)
+  (let ((inside (lvar-uses lvar)))
+    (unless (combination-p inside)
+      (give-up-ir1-transform))
+    (let ((inside-fun (combination-fun inside)))
+      (unless (member (lvar-fun-name inside-fun) (ensure-list fun))
         (give-up-ir1-transform))
-      (let ((inside-fun (combination-fun inside)))
-        (unless (member (lvar-fun-name inside-fun) fun)
+      (let ((inside-args (combination-args inside)))
+        (unless (= (length inside-args) num-args)
           (give-up-ir1-transform))
-        (let ((inside-args (combination-args inside)))
-          (unless (= (length inside-args) num-args)
-            (give-up-ir1-transform))
-          (values (lvar-fun-name inside-fun) inside-args))))))
+        (values (lvar-fun-name inside-fun) inside-args)))))
 
 (defun flush-combination (combination)
   (declare (type combination combination))
@@ -2091,6 +2073,14 @@ is :ANY, the function name is not checked."
               nil))
         nil)))
 
+;;; As above, but allow a quoted symbol also,
+;;; in which case we don't check for notinline-ness,
+;;; so be careful how you use this.
+;;; Also note that Case 2 in LVAR-FUN-IS for dealing with #.#'NAME
+;;; has no equivalent here.
+(defun lvar-fun-name* (lvar)
+  (if (constant-lvar-p lvar) (lvar-value lvar) (lvar-fun-name lvar)))
+
 (defun lvar-fun-debug-name (lvar)
   (declare (type lvar lvar))
   (let ((uses (lvar-uses lvar)))
