@@ -684,9 +684,9 @@
                               (args-type-optional type))
                       (list type))))
          (primitive-t *backend-t-primitive-type*))
-    (loop for rtype in rtypes
-          for type = (or (pop types) primitive-t)
-          collect type)))
+    (mapcar (lambda (rtype)
+              (declare (ignore rtype))
+              (or (pop types) primitive-t)) rtypes)))
 
 ;;; Return a list of TNs usable in a CALL to TEMPLATE delivering values to
 ;;; LVAR. As an efficiency hack, we pick off the common case where the LVAR is
@@ -1639,14 +1639,26 @@ not stack-allocated LVAR ~S." source-lvar)))))
 ;;; implementation.
 (defoptimizer (%special-bind ir2-convert) ((var value) node block)
   (let ((name (leaf-source-name (lvar-value var))))
-    #!-(and sb-thread x86-64)
-    (vop bind node block (lvar-tn node block value) (emit-constant name))
-    #!+(and sb-thread x86-64)
-    (progn
-      ;; GC must understand that the symbol is implicitly live even though
-      ;; binding makes no references to the object.
-      (emit-constant name)
-      (vop sb!vm::bind/let node block (lvar-tn node block value) name))))
+    ;; Emit either BIND or DYNBIND, preferring BIND if both exist.
+    ;; If only one exists, it's DYNBIND.
+    ;; Even if the backend supports load-time TLS index assignment,
+    ;; there might be only one vop (as with arm64).
+    (macrolet ((doit (bind dynbind)
+                 (if (gethash 'bind *backend-parsed-vops*) bind dynbind)))
+      (doit
+       (progn
+         ;; Inform later SYMBOL-VALUE calls that they can
+         ;; assume a nonzero tls-index.
+         ;; FIXME: setting INFO is inefficient when not actually
+         ;; changing anything
+         (unless (info :variable :wired-tls name)
+           (setf (info :variable :wired-tls name) :always-has-tls))
+         ;; We force the symbol into the code constants in case BIND
+         ;; does not actually reference it, as with x86.
+         (emit-constant name)
+         (vop bind node block (lvar-tn node block value) name))
+       (vop dynbind node block (lvar-tn node block value)
+            (emit-constant name))))))
 
 (defoptimizer (%special-unbind ir2-convert) ((var) node block)
   (declare (ignore var))
@@ -1671,7 +1683,7 @@ not stack-allocated LVAR ~S." source-lvar)))))
                               ;; CLHS says "bound and then made to have no value" -- user
                               ;; should not be able to tell the difference between that and this.
                               (about-to-modify-symbol-value var 'progv)
-                              (%primitive bind unbound-marker var))))
+                              (%primitive dynbind unbound-marker var))))
                         (,bind (vars vals)
                           (declare (optimize (speed 2) (debug 0)
                                              (insert-debug-catch 0)))
@@ -1681,7 +1693,7 @@ not stack-allocated LVAR ~S." source-lvar)))))
                                  (let ((val (car vals))
                                        (var (car vars)))
                                    (about-to-modify-symbol-value var 'progv val t)
-                                   (%primitive bind val var))
+                                   (%primitive dynbind val var))
                                  (,bind (cdr vars) (cdr vals))))))
                  (,bind ,vars ,vals))
                nil
@@ -1759,7 +1771,13 @@ not stack-allocated LVAR ~S." source-lvar)))))
   (let* ((2info (nlx-info-info info))
          (kind (cleanup-kind (nlx-info-cleanup info)))
          (block-tn (physenv-live-tn
-                    (make-normal-tn (primitive-type-or-lose 'catch-block))
+                    (make-normal-tn
+                     (primitive-type-or-lose
+                      (ecase kind
+                        (:catch
+                         'catch-block)
+                        ((:unwind-protect :block :tagbody)
+                         'unwind-block))))
                     (node-physenv node)))
          (res (make-stack-pointer-tn))
          (target-label (ir2-nlx-info-target 2info)))
