@@ -50,6 +50,9 @@
         for dx = (leaf-dynamic-extent var)
         when (and dx arg (not (lvar-dynamic-extent arg)))
         append (handle-nested-dynamic-extent-lvars dx arg) into dx-lvars
+        ;; The block may end up being deleted due to cast optimization
+        ;; caused by USE-GOOD-FOR-DX-P
+        when (node-to-be-deleted-p call) return nil
         finally (when dx-lvars
                   ;; Stack analysis requires that the CALL ends the block, so
                   ;; that MAP-BLOCK-NLXES sees the cleanup we insert here.
@@ -112,11 +115,11 @@
 (defun convert-call (ref call fun)
   (declare (type ref ref) (type combination call) (type clambda fun))
   (propagate-to-args call fun)
-  (setf (basic-combination-kind call) :local)
   (unless (call-full-like-p call)
     (dolist (arg (basic-combination-args call))
       (when arg
         (flush-lvar-externally-checkable-type arg))))
+  (setf (basic-combination-kind call) :local)
   (sset-adjoin fun (lambda-calls-or-closes (node-home-lambda call)))
   (recognize-dynamic-extent-lvars call fun)
   (merge-tail-sets call fun)
@@ -725,7 +728,9 @@
                   (:optional
                    (call-args (car temp))
                    (when (arg-info-supplied-p info)
-                     (call-args t)))
+                     (call-args (if (arg-info-supplied-used-p info)
+                                    t
+                                    1))))
                   (:rest
                    (call-args `(list ,@more-temps))
                    ;; &REST arguments may be accompanied by extra
@@ -756,7 +761,12 @@
                 (call-args temp)
                 (call-args (arg-info-default info)))
             (when (arg-info-supplied-p info)
-              (call-args (not (null temp))))))
+              (call-args (cond ((arg-info-supplied-used-p info)
+                                (not (null temp)))
+                               (temp
+                                1)
+                               (t
+                                0))))))
 
         (convert-hairy-fun-entry ref call (optional-dispatch-main-entry fun)
                                  (append temps more-temps)
