@@ -268,14 +268,15 @@
     (once-only ((n-object object))
       (ecase (numeric-type-complexp type)
         (:real
-         (if (and #!-(or x86 x86-64 arm) ;; Not implemented elsewhere yet
-                  nil
-                  (eql (numeric-type-class type) 'integer)
-                  (eql (numeric-type-low type) 0)
-                  (fixnump (numeric-type-high type)))
-             `(fixnum-mod-p ,n-object ,(numeric-type-high type))
-             `(and (typep ,n-object ',base)
-                   ,(transform-numeric-bound-test n-object type base))))
+         (cond #!+(or x86 x86-64 arm arm64) ;; Not implemented elsewhere yet
+               ((and
+                 (eql (numeric-type-class type) 'integer)
+                 (eql (numeric-type-low type) 0)
+                 (fixnump (numeric-type-high type)))
+                `(fixnum-mod-p ,n-object ,(numeric-type-high type)))
+               (t
+                `(and (typep ,n-object ',base)
+                      ,(transform-numeric-bound-test n-object type base)))))
         (:complex
          `(and (complexp ,n-object)
                ,(once-only ((n-real `(realpart (truly-the complex ,n-object)))
@@ -297,6 +298,10 @@
   (declare (type hairy-type type))
   (let ((spec (hairy-type-specifier type)))
     (cond ((unknown-type-p type)
+           #+sb-xc-host
+           (warn "can't open-code test of unknown type ~S"
+                 (type-specifier type))
+           #-sb-xc-host
            (when (policy *lexenv* (> speed inhibit-warnings))
              (compiler-notify "can't open-code test of unknown type ~S"
                               (type-specifier type)))
@@ -304,7 +309,17 @@
           (t
            (ecase (first spec)
              (satisfies
-              `(if (funcall (global-function ,(second spec)) ,object) t nil))
+              (let* ((name (second spec))
+                     (expansion (fun-name-inline-expansion name)))
+                ;; Lambda without lexenv can easily be handled here.
+                ;; This fixes the issue that LEGAL-FUN-NAME-P which is
+                ;; just a renaming of VALID-FUNCTION-NAME-P would not
+                ;; be inlined when testing the FUNCTION-NAME type.
+                `(if ,(if (and (typep expansion '(cons (eql lambda)))
+                               (not (fun-lexically-notinline-p name)))
+                          `(,expansion ,object)
+                          `(funcall (global-function ,name) ,object))
+                     t nil)))
              ((not and)
               (once-only ((n-obj object))
                 `(,(first spec) ,@(mapcar (lambda (x)
@@ -639,12 +654,8 @@
                    ;; There is no loss in the case where both fail, and there
                    ;; is a benefit in a passing case. Always try both though,
                    ;; because (MAKE-INSTANCE 'x) works on any structure class.
-                   (abstract-base-p
-                    (let ((dd (layout-info layout)))
-                      (and dd
-                           (not (dd-default-constructor dd))
-                           (let ((ctors (dd-constructors dd)))
-                             (or (not ctors) (equal ctors '((nil))))))))
+                   (abstract-base-p (awhen (layout-info layout)
+                                      (not (dd-constructors it))))
                    (get-ancestor
                     ;; Use DATA-VECTOR-REF directly, since that's what SVREF in
                     ;; a SAFETY 0 lexenv will eventually be transformed to.
@@ -796,28 +807,63 @@
         t))))
 
 ;;; Drops dimension information from vector types.
+;;; Returns four values
+;;; * vector ctype
+;;; * upgraded-element ctype or requsted element
+;;; * T if the upgraded-element is upgraded, i.e. it
+;;;   does not contain any unknown types.
+;;; * T if there were any dimensions
 (defun simplify-vector-type (type)
-  (aver (csubtypep type (specifier-type '(array * (*)))))
-  (let* ((array-type
-          (if (csubtypep type (specifier-type 'simple-array))
-              'simple-array
-              'array))
-         (complexp
-          (not
-           (or (eq 'simple-array array-type)
-               (neq *empty-type*
-                    (type-intersection type (specifier-type 'simple-array)))))))
-    (dolist (etype
-              #+sb-xc-host '(t bit character)
-              #-sb-xc-host sb!kernel::*specialized-array-element-types*
-              #+sb-xc-host (values nil nil nil)
-              #-sb-xc-host (values `(,array-type * (*)) t complexp))
-      (when etype
-        (let ((simplified (specifier-type `(,array-type ,etype (*)))))
-          (when (csubtypep type simplified)
-            (return (values (type-specifier simplified)
-                            etype
-                            complexp))))))))
+  (labels ((process-compound-type (types)
+             (let (array-types
+                   element-types
+                   (upgraded t)
+                   dimensions-removed)
+               (dolist (type types)
+                 (unless (or (hairy-type-p type)
+                             (sb!kernel::negation-type-p type))
+                   (multiple-value-bind (type et upgraded dimensions) (simplify type)
+                     (push type array-types)
+                     (push et element-types)
+                     (when dimensions
+                       (setf dimensions-removed t))
+                     (unless upgraded
+                       (setf upgraded nil)))))
+               (values (apply #'type-union array-types)
+                       (apply #'type-union element-types)
+                       upgraded
+                       dimensions-removed)))
+           (simplify (type)
+             (cond ((and (array-type-p type)
+                         (singleton-p (array-type-dimensions type)))
+                    (let* ((upgraded t)
+                           (et (array-type-specialized-element-type type))
+                           (et (cond ((neq et *wild-type*)
+                                      et)
+                                     ((eq (array-type-element-type type) *wild-type*)
+                                      et)
+                                     (t
+                                      (setf upgraded nil)
+                                      (array-type-element-type type)))))
+                      (values (specifier-type
+                               (list (if (array-type-complexp type)
+                                         'array
+                                         'simple-array)
+                                     (type-specifier et)
+                                     '(*)))
+                              et
+                              upgraded
+                              (not (eq (car (array-type-dimensions type)) '*)))))
+                   ((union-type-p type)
+                    (process-compound-type (union-type-types type)))
+                   ((intersection-type-p type)
+                    (process-compound-type (intersection-type-types type)))
+                   ((member-type-p type)
+                    (process-compound-type
+                     (mapcar #'ctype-of (member-type-members type))))
+                   (t
+                    (error "~a is not a subtype of VECTOR." type)))))
+    (simplify type)))
 
 (deftransform coerce ((x type) (* *) * :node node)
   (unless (constant-lvar-p type)
@@ -855,8 +901,8 @@
                    (the ,result-type
                         (complex (coerce (realpart x) ',part-type)
                                  (coerce (imagpart x) ',part-type))))))))
-           ;; Special case STRING and SIMPLE-STRING as they are union types
-           ;; in SBCL.
+          ;; Special case STRING and SIMPLE-STRING as they are union types
+          ;; in SBCL.
           ((member tval '(string simple-string))
            `(the ,tval
                  (if (typep x ',tval)
@@ -864,50 +910,51 @@
                      (replace (make-array (length x) :element-type 'character) x))))
           ((eq tval 'character)
            `(character x))
-           ;; Special case VECTOR
-           ((eq tval 'vector)
-            `(the ,tval
-               (if (vectorp x)
-                   x
-                   (replace (make-array (length x)) x))))
-           ;; Handle specialized element types for 1D arrays.
-           ((csubtypep tspec (specifier-type '(array * (*))))
-            ;; Can we avoid checking for dimension issues like (COERCE FOO
-            ;; '(SIMPLE-VECTOR 5)) returning a vector of length 6?
-            ;;
-            ;; CLHS actually allows this for all code with SAFETY < 3,
-            ;; but we're a conservative bunch.
-            (if (or (policy node (zerop safety)) ; no need in unsafe code
-                    (and (array-type-p tspec)    ; no need when no dimensions
-                         (equal (array-type-dimensions tspec) '(*))))
-                ;; We can!
-                (multiple-value-bind (vtype etype complexp) (simplify-vector-type tspec)
-                  (unless vtype
-                    (give-up-ir1-transform))
-                  `(the ,vtype
-                     (if (typep x ',vtype)
-                         x
-                         (replace
-                          (make-array (length x) :element-type ',etype
-                                      ,@(when complexp
-                                              (list :fill-pointer t
-                                                    :adjustable t)))
-                          x))))
-                ;; No, duh. Dimension checking required.
-                (give-up-ir1-transform
-                 "~@<~S specifies dimensions other than (*) in safe code.~:@>"
-                 tval)))
-           ((type= tspec (specifier-type 'list))
-            `(coerce-to-list x))
-           ((csubtypep tspec (specifier-type 'function))
-            (if (csubtypep (lvar-type x) (specifier-type 'symbol))
-                `(coerce-symbol-to-fun x)
-                ;; if X can later be derived as FUNCTION then we don't want
-                ;; to call COERCE-TO-FUN, because there's no smartness
-                ;; that can undo that and see that it's really (IDENTITY X).
-                (progn (delay-ir1-transform node :constraint)
-                       `(coerce-to-fun x))))
-           (t
-            (give-up-ir1-transform
-             "~@<open coding coercion to ~S not implemented.~:@>"
-             tval))))))
+          ;; Special case VECTOR
+          ((eq tval 'vector)
+           `(the ,tval
+                 (if (vectorp x)
+                     x
+                     (replace (make-array (length x)) x))))
+          ;; Handle specialized element types for 1D arrays.
+          ((csubtypep tspec (specifier-type '(array * (*))))
+           ;; Can we avoid checking for dimension issues like (COERCE FOO
+           ;; '(SIMPLE-VECTOR 5)) returning a vector of length 6?
+           ;;
+           ;; CLHS actually allows this for all code with SAFETY < 3,
+           ;; but we're a conservative bunch.
+           (if (or (policy node (zerop safety)) ; no need in unsafe code
+                   (and (array-type-p tspec) ; no need when no dimensions
+                        (equal (array-type-dimensions tspec) '(*))))
+               ;; We can!
+               (multiple-value-bind (vtype etype upgraded) (simplify-vector-type tspec)
+                 (unless upgraded
+                   (give-up-ir1-transform))
+                 (let ((vtype (type-specifier vtype)))
+                   `(the ,vtype
+                         (if (typep x ',vtype)
+                             x
+                             (replace
+                              (make-array (length x)
+                                          ,@(and (not (eq etype *universal-type*))
+                                                 (not (eq etype *wild-type*))
+                                                 `(:element-type ',(type-specifier etype))))
+                              x)))))
+               ;; No, duh. Dimension checking required.
+               (give-up-ir1-transform
+                "~@<~S specifies dimensions other than (*) in safe code.~:@>"
+                tval)))
+          ((type= tspec (specifier-type 'list))
+           `(coerce-to-list x))
+          ((csubtypep tspec (specifier-type 'function))
+           (if (csubtypep (lvar-type x) (specifier-type 'symbol))
+               `(coerce-symbol-to-fun x)
+               ;; if X can later be derived as FUNCTION then we don't want
+               ;; to call COERCE-TO-FUN, because there's no smartness
+               ;; that can undo that and see that it's really (IDENTITY X).
+               (progn (delay-ir1-transform node :constraint)
+                      `(coerce-to-fun x))))
+          (t
+           (give-up-ir1-transform
+            "~@<open coding coercion to ~S not implemented.~:@>"
+            tval))))))
