@@ -25,87 +25,87 @@
 ;;;; machine architecture parameters
 
 ;;; the number of bits per word, where a word holds one lisp descriptor
-(def!constant n-word-bits 32)
+(defconstant n-word-bits 32)
 
 ;;; the natural width of a machine word (as seen in e.g. register width,
 ;;; address space)
-(def!constant n-machine-word-bits 32)
+(defconstant n-machine-word-bits 32)
 
 ;;; the number of bits per byte, where a byte is the smallest
 ;;; addressable object
-(def!constant n-byte-bits 8)
+(defconstant n-byte-bits 8)
 
 ;;; The minimum immediate offset in a memory-referencing instruction.
-(def!constant minimum-immediate-offset (- (expt 2 31)))
+(defconstant minimum-immediate-offset (- (expt 2 31)))
 
 ;;; The maximum immediate offset in a memory-referencing instruction.
-(def!constant maximum-immediate-offset (1- (expt 2 31)))
+(defconstant maximum-immediate-offset (1- (expt 2 31)))
 
-(def!constant float-sign-shift 31)
+(defconstant float-sign-shift 31)
 
 ;;; comment from CMU CL:
 ;;;   These values were taken from the alpha code. The values for
 ;;;   bias and exponent min/max are not the same as shown in the 486 book.
 ;;;   They may be correct for how Python uses them.
-(def!constant single-float-bias 126)    ; Intel says 127.
+(defconstant single-float-bias 126)    ; Intel says 127.
 (defconstant-eqx single-float-exponent-byte    (byte 8 23) #'equalp)
 (defconstant-eqx single-float-significand-byte (byte 23 0) #'equalp)
 ;;; comment from CMU CL:
 ;;;   The 486 book shows the exponent range -126 to +127. The Lisp
 ;;;   code that uses these values seems to want already biased numbers.
-(def!constant single-float-normal-exponent-min 1)
-(def!constant single-float-normal-exponent-max 254)
-(def!constant single-float-hidden-bit (ash 1 23))
-(def!constant single-float-trapping-nan-bit (ash 1 22))
+(defconstant single-float-normal-exponent-min 1)
+(defconstant single-float-normal-exponent-max 254)
+(defconstant single-float-hidden-bit (ash 1 23))
+(defconstant single-float-trapping-nan-bit (ash 1 22))
 
-(def!constant double-float-bias 1022)
+(defconstant double-float-bias 1022)
 (defconstant-eqx double-float-exponent-byte    (byte 11 20) #'equalp)
 (defconstant-eqx double-float-significand-byte (byte 20 0)  #'equalp)
-(def!constant double-float-normal-exponent-min 1)
-(def!constant double-float-normal-exponent-max #x7FE)
-(def!constant double-float-hidden-bit (ash 1 20))
-(def!constant double-float-trapping-nan-bit (ash 1 19))
+(defconstant double-float-normal-exponent-min 1)
+(defconstant double-float-normal-exponent-max #x7FE)
+(defconstant double-float-hidden-bit (ash 1 20))
+(defconstant double-float-trapping-nan-bit (ash 1 19))
 
-(def!constant long-float-bias 16382)
+(defconstant long-float-bias 16382)
 (defconstant-eqx long-float-exponent-byte    (byte 15 0) #'equalp)
 (defconstant-eqx long-float-significand-byte (byte 31 0) #'equalp)
-(def!constant long-float-normal-exponent-min 1)
-(def!constant long-float-normal-exponent-max #x7FFE)
-(def!constant long-float-hidden-bit (ash 1 31))         ; actually not hidden
-(def!constant long-float-trapping-nan-bit (ash 1 30))
+(defconstant long-float-normal-exponent-min 1)
+(defconstant long-float-normal-exponent-max #x7FFE)
+(defconstant long-float-hidden-bit (ash 1 31))         ; actually not hidden
+(defconstant long-float-trapping-nan-bit (ash 1 30))
 
-(def!constant single-float-digits
+(defconstant single-float-digits
   (+ (byte-size single-float-significand-byte) 1))
 
-(def!constant double-float-digits
+(defconstant double-float-digits
   (+ (byte-size double-float-significand-byte) n-word-bits 1))
 
-(def!constant long-float-digits
+(defconstant long-float-digits
   (+ (byte-size long-float-significand-byte) n-word-bits 1))
 
 ;;; pfw -- from i486 microprocessor programmer's reference manual
-(def!constant float-invalid-trap-bit       (ash 1 0))
-(def!constant float-denormal-trap-bit       (ash 1 1))
-(def!constant float-divide-by-zero-trap-bit (ash 1 2))
-(def!constant float-overflow-trap-bit       (ash 1 3))
-(def!constant float-underflow-trap-bit      (ash 1 4))
-(def!constant float-inexact-trap-bit       (ash 1 5))
+(defconstant float-invalid-trap-bit       (ash 1 0))
+(defconstant float-denormal-trap-bit       (ash 1 1))
+(defconstant float-divide-by-zero-trap-bit (ash 1 2))
+(defconstant float-overflow-trap-bit       (ash 1 3))
+(defconstant float-underflow-trap-bit      (ash 1 4))
+(defconstant float-inexact-trap-bit       (ash 1 5))
 
-(def!constant float-round-to-nearest  0)
-(def!constant float-round-to-negative 1)
-(def!constant float-round-to-positive 2)
-(def!constant float-round-to-zero     3)
+(defconstant float-round-to-nearest  0)
+(defconstant float-round-to-negative 1)
+(defconstant float-round-to-positive 2)
+(defconstant float-round-to-zero     3)
 
-(def!constant float-precision-24-bit 0)
-(def!constant float-precision-53-bit 2)
-(def!constant float-precision-64-bit 3)
+(defconstant float-precision-24-bit 0)
+(defconstant float-precision-53-bit 2)
+(defconstant float-precision-64-bit 3)
 
 (defconstant-eqx float-rounding-mode     (byte 2 10) #'equalp)
 (defconstant-eqx float-sticky-bits       (byte 6 16) #'equalp)
 (defconstant-eqx float-traps-byte        (byte 6  0) #'equalp)
 (defconstant-eqx float-exceptions-byte   (byte 6 16) #'equalp)
 (defconstant-eqx float-precision-control (byte 2  8) #'equalp)
-(def!constant float-fast-bit 0) ; no fast mode on x86
+(defconstant float-fast-bit 0) ; no fast mode on x86
 
 ;;;; description of the target address space
 
@@ -197,11 +197,11 @@
 #!+darwin    (!gencgc-space-setup #x04000000 :dynamic-space-start #x10000000)
 
 ;;; Size of one linkage-table entry in bytes.
-(def!constant linkage-table-entry-size 8)
+(defconstant linkage-table-entry-size 8)
 
 ;;; Given that NIL is the first thing allocated in static space, we
 ;;; know its value at compile time:
-(def!constant nil-value (+ static-space-start #xb))
+(defconstant nil-value (+ static-space-start #xb))
 
 ;;;; other miscellaneous constants
 
