@@ -60,17 +60,27 @@
 
     DONE
     (inst movzx result al-tn)))
+
+(define-vop (%other-pointer-widetag)
+  (:translate %other-pointer-widetag)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg)))
+  (:results (result :scs (unsigned-reg)))
+  (:result-types positive-fixnum)
+  (:generator 6
+    (inst movzx result (make-ea :byte :base object
+                                      :disp (- other-pointer-lowtag)))))
+
 
 (define-vop (fun-subtype)
   (:translate fun-subtype)
   (:policy :fast-safe)
   (:args (function :scs (descriptor-reg)))
-  (:temporary (:sc byte-reg :from (:eval 0) :to (:eval 1)) temp)
   (:results (result :scs (unsigned-reg)))
   (:result-types positive-fixnum)
   (:generator 6
-    (load-type temp function (- fun-pointer-lowtag))
-    (inst movzx result temp)))
+    (inst movzx result (make-ea :byte :base function
+                                      :disp (- fun-pointer-lowtag)))))
 
 (define-vop (set-fun-subtype)
   (:translate (setf fun-subtype))
@@ -120,7 +130,7 @@
   (:generator 6
     (move eax data)
     (inst shl eax (- n-widetag-bits 2))
-    (inst mov al-tn (make-ea :byte :base x :disp (- other-pointer-lowtag)))
+    (load-type al-tn x (- other-pointer-lowtag))
     (storew eax x 0 other-pointer-lowtag)
     (move res x)))
 
@@ -233,6 +243,43 @@
                                             fun-pointer-lowtag))
     (storew temp function simple-fun-self-slot fun-pointer-lowtag)
     (move result new-self)))
+
+;;;; symbol frobbing
+
+;; only define if the feature is enabled to test building without it
+#!+symbol-info-vops
+(progn
+(define-vop (symbol-info-vector)
+  (:policy :fast-safe)
+  (:translate symbol-info-vector)
+  (:args (x :scs (descriptor-reg)))
+  (:results (res :scs (descriptor-reg)))
+  (:temporary (:sc unsigned-reg :offset eax-offset) eax)
+  (:generator 1
+    (loadw res x symbol-info-slot other-pointer-lowtag)
+    ;; If RES has list-pointer-lowtag, take its CDR. If not, use it as-is.
+    ;; This CMOV safely reads from memory when it does not move, because if
+    ;; there is an info-vector in the slot, it has at least one element.
+    ;; This would compile to almost the same code without a VOP,
+    ;; but using a jmp around a mov instead.
+    (inst lea eax (make-ea :dword :base res :disp (- list-pointer-lowtag)))
+    (emit-optimized-test-inst eax lowtag-mask)
+    (inst cmov :e res
+          (make-ea-for-object-slot res cons-cdr-slot list-pointer-lowtag))))
+(define-vop (symbol-plist)
+  (:policy :fast-safe)
+  (:translate symbol-plist)
+  (:args (x :scs (descriptor-reg)))
+  (:results (res :scs (descriptor-reg)))
+  (:temporary (:sc unsigned-reg) temp)
+  (:generator 1
+    (loadw res x symbol-info-slot other-pointer-lowtag)
+    ;; Instruction pun: (CAR x) is the same as (VECTOR-LENGTH x)
+    ;; so if the info slot holds a vector, this gets a fixnum- it's not a plist.
+    (loadw res res cons-car-slot list-pointer-lowtag)
+    (inst mov temp nil-value)
+    (emit-optimized-test-inst res fixnum-tag-mask)
+    (inst cmov :e res temp))))
 
 ;;;; other miscellaneous VOPs
 
@@ -399,11 +446,11 @@ number of CPU cycles elapsed as secondary value. EXPERIMENTAL."
 
 ;;;;
 
-(defknown %cons-cas-pair (cons t t t t) (values t t &optional))
+(defknown %cons-cas-pair (cons t t t t) (values t t))
 ;; These unsafely permits cmpxchg on any kind of vector, boxed or unboxed
 ;; and the same goes for instances.
-(defknown %vector-cas-pair (simple-array index t t t t) (values t t &optional))
-(defknown %instance-cas-pair (instance index t t t t) (values t t &optional))
+(defknown %vector-cas-pair (simple-array index t t t t) (values t t))
+(defknown %instance-cas-pair (instance index t t t t) (values t t))
 
 (macrolet
     ((define-cmpxchg-vop (name memory-operand more-stuff &optional index-arg)
