@@ -11,7 +11,7 @@
 
 (in-package "SB!C")
 
-(defvar *xref-kinds* '(:binds :calls :sets :references :macroexpands))
+(defglobal *xref-kinds* '(:binds :calls :sets :references :macroexpands))
 
 (defun record-component-xrefs (component)
   (declare (type component component))
@@ -28,20 +28,20 @@
                (loop for ctran = start then (node-next (ctran-next ctran))
                      while ctran
                      do (record-node-xrefs (ctran-next ctran) functional))
-               ;; Properly record the deferred macroexpansion information
-               ;; that's been stored in the block.
-               (dolist (xref-data (block-macroexpands block))
-                 (record-xref :macroexpands
-                              (car xref-data)
-                              ;; We use the debug-name of the functional
-                              ;; as an identifier. This works quite nicely,
-                              ;; except for (fast/slow)-methods with non-symbol,
-                              ;; non-number eql specializers, for which
-                              ;; the debug-name doesn't map exactly
-                              ;; to the fdefinition of the method.
-                              functional
-                              nil
-                              (cdr xref-data)))))
+               ;; Properly record the deferred macroexpansion and source
+               ;; transform information that's been stored in the block.
+               (dolist (xref-data (block-xrefs block))
+                 (destructuring-bind (kind what path) xref-data
+                   (record-xref kind what
+                                ;; We use the debug-name of the functional
+                                ;; as an identifier. This works quite nicely,
+                                ;; except for (fast/slow)-methods with non-symbol,
+                                ;; non-number eql specializers, for which
+                                ;; the debug-name doesn't map exactly
+                                ;; to the fdefinition of the method.
+                                functional
+                                nil
+                                path)))))
         (call-with-block-external-functionals block #'handle-node)))))
 
 (defun call-with-block-external-functionals (block fun)
@@ -94,36 +94,25 @@
           (let* ((name (leaf-debug-name leaf)))
             (case (global-var-kind leaf)
               ;; Reading a special
-              (:special
+              ((:special :global)
                (record-xref :references name context node nil))
               ;; Calling a function
               (:global-function
                (record-xref :calls name context node nil)))))
          ;; Inlined global function
          (clambda
-          (when (functional-inlinep leaf)
-            (let ((name (leaf-debug-name leaf)))
-              ;; FIXME: we should store the original var into the
-              ;; functional when creating inlined-functionals, so that
-              ;; we could just check whether it was a global-var,
-              ;; rather then needing to guess based on the debug-name.
-              (when (or (symbolp name)
-                        ;; Any non-SETF non-symbol names will
-                        ;; currently be either non-functions or
-                        ;; internals.
-                        (and (consp name)
-                             (equal (car name) 'setf)))
-                ;; TODO: a WHO-INLINES xref-kind could be useful
-                (record-xref :calls name context node nil)))))
+          (let ((inline-var (functional-inline-expanded leaf)))
+            (when (global-var-p inline-var)
+              ;; TODO: a WHO-INLINES xref-kind could be useful
+              (record-xref :calls (leaf-debug-name inline-var) context node nil))))
          ;; Reading a constant
          (constant
-          (let* ((name (constant-%source-name leaf)))
-            (record-xref :references name context node nil))))))
+          (record-xref :references (ref-%source-name node) context node nil)))))
     ;; Setting a special variable
     (cset
-     (let* ((var (set-var node)))
+     (let ((var (set-var node)))
        (when (and (global-var-p var)
-                  (eq :special (global-var-kind var)))
+                  (memq (global-var-kind var) '(:special :global)))
          (record-xref :sets
                       (leaf-debug-name var)
                       context
@@ -141,37 +130,48 @@
                         nil)))))))
 
 (defun internal-name-p (what)
-  ;; Don't store XREF information for internals. We define as internal
-  ;; anything named only by symbols from either implementation
-  ;; packages, COMMON-LISP or KEYWORD. The last one is useful for
-  ;; example when dealing with ctors.
+  ;; Unless we're building with SB-XREF-FOR-INTERNALS, don't store
+  ;; XREF information for internals. We define anything with a symbol
+  ;; from either an implementation package or from COMMON-LISP as
+  ;; internal
   (typecase what
     (list
      (every #'internal-name-p what))
     (symbol
-     (member (symbol-package what)
-             (load-time-value (list* (find-package "COMMON-LISP")
-                                     (find-package "KEYWORD")
-                                     (remove-if-not
-                                      (lambda (package)
-                                        (= (mismatch "SB!"
-                                                     (package-name package))
-                                           3))
-                                      (list-all-packages))))))
+     #!+sb-xref-for-internals
+     (eq '.anonymous. what)
+     #!-sb-xref-for-internals
+     (or (eq '.anonymous. what)
+         (member (symbol-package what)
+                 (load-time-value
+                  (list* (find-package "COMMON-LISP")
+                         #+sb-xc-host (find-package "SB-XC")
+                         (remove-if-not
+                          (lambda (package)
+                            (= (mismatch #+sb-xc "SB-" #-sb-xc "SB!"
+                                         (package-name package))
+                               3))
+                          (list-all-packages))) t))
+         #+sb-xc-host   ; again, special case like in genesis and dump
+         (multiple-value-bind (cl-symbol cl-status)
+             (find-symbol (symbol-name what) sb!int:*cl-package*)
+           (and (eq what cl-symbol) (eq cl-status :external)))))
     (t t)))
 
 (defun record-xref (kind what context node path)
   (unless (internal-name-p what)
-    (let ((path (reverse
-                 (source-path-original-source
-                  (or path
-                      (node-source-path node))))))
-      (push (list what path)
-            (getf (functional-xref context) kind)))))
+    (push (cons what
+                (source-path-form-number (or path
+                                             (node-source-path node))))
+          (getf (functional-xref context) kind))))
 
 (defun record-macroexpansion (what block path)
   (unless (internal-name-p what)
-    (push (cons what path) (block-macroexpands block))))
+    (push (list :macroexpands what path) (block-xrefs block))))
+
+(defun record-call (what block path)
+  (unless (internal-name-p what)
+    (push (list :calls what path) (block-xrefs block))))
 
 ;;; Pack the xref table that was stored for a functional into a more
 ;;; space-efficient form, and return that packed form.
@@ -182,9 +182,14 @@
             for i from 0
             for values = (remove-duplicates (getf xref-data key)
                                             :test #'equal)
-            for flattened = (reduce #'append values :from-end t)
-            collect (setf (aref array i)
-                          (when flattened
-                            (make-array (length flattened)
-                                        :initial-contents flattened))))
+            do
+            (setf (aref array i)
+                  (when values
+                    (let* ((length (* (length values) 2))
+                           (data (make-array length)))
+                      (loop for i below length by 2
+                            for (name . number) in values
+                            do (setf (aref data i) name
+                                     (aref data (1+ i)) number))
+                      data))))
       array)))
