@@ -34,9 +34,6 @@
 ;;;; very low-level representation of instances with meta-class
 ;;;; STANDARD-CLASS
 
-(defknown sb-pcl::pcl-instance-p (t) boolean
-  (movable foldable flushable explicit-check))
-
 (deftransform sb-pcl::pcl-instance-p ((object))
   (let* ((otype (lvar-type object))
          (standard-object (specifier-type 'standard-object)))
@@ -45,13 +42,12 @@
       ((csubtypep otype standard-object) t)
       ((not (types-equal-or-intersect otype standard-object)) nil)
       (t
-       `(layout-for-std-class-p (layout-of object))))))
+       `(sb-pcl::%pcl-instance-p object)))))
 
 (defun sb-pcl::safe-code-p (&optional env)
-  (let* ((lexenv (or env (make-null-lexenv)))
-         (policy (lexenv-policy lexenv)))
-    (eql (cdr (assoc 'safety policy)) 3)))
+  (policy (or env (make-null-lexenv)) (eql safety 3)))
 
+(declaim (ftype function sb-pcl::parse-specialized-lambda-list))
 (define-source-context defmethod (name &rest stuff)
   (let ((arg-pos (position-if #'listp stuff)))
     (if arg-pos
@@ -83,6 +79,7 @@
 (define-internal-pcl-function-name-syntax sb-pcl::slow-method (list)
   (valid-function-name-p (cadr list)))
 
+(declaim (ftype function sb-pcl::std-instance-p sb-pcl::fsc-instance-p))
 (define-internal-pcl-function-name-syntax sb-pcl::ctor (list)
   (let ((class-or-name (cadr list)))
     (cond
@@ -92,3 +89,43 @@
            (sb-pcl::fsc-instance-p class-or-name))
        (values t nil)))))
 
+(defun interned-symbol-p (x) (and (symbolp x) (symbol-package x)))
+
+(flet ((struct-accessor-p (object slot-name)
+         (let ((c-slot-name (lvar-value slot-name)))
+           (unless (interned-symbol-p c-slot-name)
+             (give-up-ir1-transform "slot name is not an interned symbol"))
+           (let* ((type (lvar-type object))
+                  (dd (when (structure-classoid-p type)
+                        (find-defstruct-description
+                         (sb-kernel::structure-classoid-name type)))))
+              (when dd
+                (find c-slot-name (dd-slots dd) :key #'dsd-name))))))
+
+  (deftransform slot-boundp ((object slot-name) (t (constant-arg symbol)) *
+                             :node node)
+    (cond ((struct-accessor-p object slot-name) t) ; always boundp
+          (t (delay-ir1-transform node :constraint)
+             `(sb-pcl::accessor-slot-boundp object ',(lvar-value slot-name)))))
+
+  (deftransform slot-value ((object slot-name) (t (constant-arg symbol)) *
+                            :node node)
+    (acond ((struct-accessor-p object slot-name)
+            `(,(dsd-accessor-name it) object))
+           (t
+            (delay-ir1-transform node :constraint)
+            `(sb-pcl::accessor-slot-value object ',(lvar-value slot-name)))))
+
+  (deftransform sb-pcl::set-slot-value ((object slot-name new-value)
+                                        (t (constant-arg symbol) t)
+                                        * :node node)
+    (acond ((struct-accessor-p object slot-name)
+            `(setf (,(dsd-accessor-name it) object) new-value))
+           ((policy node (= safety 3))
+            ;; Safe code wants to check the type, and the global
+            ;; accessor won't do that.
+            (give-up-ir1-transform "cannot use optimized accessor in safe code"))
+           (t
+            (delay-ir1-transform node :constraint)
+            `(sb-pcl::accessor-set-slot-value object ',(lvar-value slot-name)
+                                              new-value)))))
