@@ -123,8 +123,10 @@
 ;;; When the optimized function is computed, the function of the
 ;;; funcallable instance is set to it.
 ;;;
+
+;;; Type is either CTOR, for MAKE-INSTANCE, or ALLOCATOR, for ALLOCATE-INSTANCE
 (!defstruct-with-alternate-metaclass ctor
-  :slot-names (function-name class-or-name class initargs state safe-p)
+  :slot-names (type function-name class-or-name class initargs state safe-p)
   :boa-constructor %make-ctor
   :superclass-name function
   :metaclass-name static-classoid
@@ -145,11 +147,15 @@
     (setf (ctor-class ctor) nil
           (ctor-state ctor) 'initial)
     (setf (funcallable-instance-fun ctor)
-          #'(lambda (&rest args)
-              (install-optimized-constructor ctor)
-              (apply ctor args)))
-    (setf (%funcallable-instance-info ctor 1)
-          (ctor-function-name ctor))))
+          (ecase (ctor-type ctor)
+            (ctor
+             (lambda (&rest args)
+               (install-optimized-constructor ctor)
+               (apply ctor args)))
+            (allocator
+             (lambda ()
+               (install-optimized-allocator ctor)
+               (funcall ctor)))))))
 
 (defun make-ctor-function-name (class-name initargs safe-code-p)
   (labels ((arg-name (x)
@@ -178,7 +184,21 @@
 ;;; Keep this a separate function for testing.
 (defun make-ctor (function-name class-name initargs safe-p)
   (without-package-locks ; for (setf symbol-function)
-    (let ((ctor (%make-ctor function-name class-name nil initargs nil safe-p)))
+    (let ((ctor (%make-ctor 'ctor function-name class-name nil initargs nil safe-p)))
+      (install-initial-constructor ctor :force-p t)
+      (push ctor *all-ctors*)
+      (setf (fdefinition function-name) ctor)
+      ctor)))
+
+(defun ensure-allocator (function-name class-name)
+  (with-world-lock ()
+    (if (fboundp function-name)
+        (the ctor (fdefinition function-name))
+        (make-allocator function-name class-name))))
+
+(defun make-allocator (function-name class-name)
+  (without-package-locks ; for (setf symbol-function)
+    (let ((ctor (%make-ctor 'allocator function-name class-name nil nil nil nil)))
       (install-initial-constructor ctor :force-p t)
       (push ctor *all-ctors*)
       (setf (fdefinition function-name) ctor)
@@ -218,8 +238,8 @@
 (declaim (inline sxhash-symbol-or-class))
 (defun sxhash-symbol-or-class (x)
   (cond ((symbolp x) (sxhash x))
-        ((std-instance-p x) (std-instance-hash x))
-        ((fsc-instance-p x) (fsc-instance-hash x))
+        ((std-instance-p x) (sb-impl::std-instance-hash x))
+        ((fsc-instance-p x) (sb-impl::fsc-instance-hash x))
         (t
          (bug "Something strange where symbol or class expected."))))
 
@@ -354,6 +374,34 @@
          (if ctor
              (values ctor store)
              (put-ctor (maybe-ctor-for-caching) store))))))
+
+(defun ensure-cached-allocator (class store)
+  (flet ((maybe-ctor-for-caching ()
+           (if (classp class)
+               (let ((function-name (list 'ctor 'allocator class)))
+                 (declare (dynamic-extent function-name))
+                 (with-world-lock ()
+                   (if (fboundp function-name)
+                       (the ctor (fdefinition function-name))
+                       (make-allocator (copy-list function-name) class))))
+               ;; Invalid first argument: let ALLOCATE-INSTANCE worry about it.
+               (return-from ensure-cached-allocator
+                 (values (lambda ()
+                           (declare (notinline allocate-instance))
+                           (allocate-instance class))
+                         store)))))
+    (if (listp store)
+        (multiple-value-bind (ctor list) (find-ctor class store)
+          (if ctor
+              (values ctor list)
+              (let ((ctor (maybe-ctor-for-caching)))
+                (if (< (length list) +ctor-list-max-size+)
+                    (values ctor (cons ctor list))
+                    (values ctor (ctor-list-to-table list))))))
+        (let ((ctor (get-ctor class store)))
+          (if ctor
+              (values ctor store)
+              (put-ctor (maybe-ctor-for-caching) store))))))
 
 ;;; ***********************************************
 ;;; Compile-Time Expansion of MAKE-INSTANCE *******
@@ -361,6 +409,9 @@
 
 (defvar *compiling-optimized-constructor* nil)
 
+;;; There are some MAKE-INSTANCE calls compiled prior to this macro definition.
+;;; While it would be trivial to move earlier, I'm not sure that it would
+;;; actually work.
 (define-compiler-macro make-instance (&whole form &rest args &environment env)
   (declare (ignore args))
   ;; Compiling an optimized constructor for a non-standard class means
@@ -370,6 +421,59 @@
         (make-instance->constructor-call form (safe-code-p env)))
       form))
 
+(define-compiler-macro allocate-instance (&whole form class &rest initargs)
+  (or (unless (or *compiling-optimized-constructor*
+                  initargs)
+        (allocate-instance->constructor-call class))
+      form))
+
+(defun proclaim-constructor (function-name)
+  ;; Prevent compiler warnings for calling the ctor.
+  (proclaim-as-fun-name function-name)
+  (note-name-defined function-name :function)
+  (when (eq (info :function :where-from function-name) :assumed)
+    (setf (info :function :where-from function-name) :defined)
+    (when (info :function :assumed-type function-name)
+      (setf (info :function :assumed-type function-name) nil))))
+
+(defun allocate-instance->constructor-call (class-arg)
+  (let ((constant-class (if (classp class-arg)
+                            class-arg
+                            (and (proper-list-of-length-p class-arg 2)
+                                 (eq (car class-arg) 'find-class)
+                                 (proper-list-of-length-p (cadr class-arg) 2)
+                                 (eq (caadr class-arg) 'quote)
+                                 (symbolp (cadadr class-arg))
+                                 (cadadr class-arg)))))
+    (if constant-class
+        (let* ((class-or-name constant-class)
+               (function-name (list 'ctor 'allocator class-or-name)))
+          (sb-int:check-deprecated-type (if (classp class-or-name)
+                                            (class-name class-or-name)
+                                            class-or-name))
+          (proclaim-constructor function-name)
+          ;; Return code constructing a ctor at load time, which,
+          ;; when called, will set its funcallable instance
+          ;; function to an optimized constructor function.
+          `(locally
+               (declare (disable-package-locks ,function-name))
+             (load-time-value
+              (ensure-allocator ',function-name ',class-or-name))
+             (funcall (function ,function-name))))
+        `(locally (declare (disable-package-locks .cache. .class-arg. .store. .fun.))
+           (let* ((.cache. (load-time-value (cons 'ctor-cache nil)))
+                  (.store. (cdr .cache.))
+                  (.class-arg. ,class-arg))
+             (multiple-value-bind (.fun. .new-store.)
+                 (ensure-cached-allocator .class-arg. .store.)
+               ;; Thread safe: if multiple threads hit this in
+               ;; parallel, the update from the other one is
+               ;; just lost -- no harm done, except for the need
+               ;; to redo the work next time.
+               (unless (eq .store. .new-store.)
+                 (setf (cdr .cache.) .new-store.))
+               (funcall (truly-the function .fun.))))))))
+
 (defun make-instance->constructor-call (form safe-code-p)
   (destructuring-bind (class-arg &rest args) (cdr form)
     (flet (;;
@@ -410,13 +514,7 @@
               (sb-int:check-deprecated-type (if (classp class-or-name)
                                                 (class-name class-or-name)
                                                 class-or-name))
-              ;; Prevent compiler warnings for calling the ctor.
-              (proclaim-as-fun-name function-name)
-              (note-name-defined function-name :function)
-              (when (eq (info :function :where-from function-name) :assumed)
-                (setf (info :function :where-from function-name) :defined)
-                (when (info :function :assumed-type function-name)
-                  (setf (info :function :assumed-type function-name) nil)))
+              (proclaim-constructor function-name)
               ;; Return code constructing a ctor at load time, which,
               ;; when called, will set its funcallable instance
               ;; function to an optimized constructor function.
@@ -489,6 +587,47 @@
                locations)
               (ctor-state ctor) (if optimizedp 'optimized 'fallback))))))
 
+(defun install-optimized-allocator (ctor)
+  (with-world-lock ()
+    (let* ((class-or-name (ctor-class-or-name ctor))
+           (class (ensure-class-finalized
+                   (if (symbolp class-or-name)
+                       (find-class class-or-name)
+                       class-or-name))))
+      ;; We can have a class with an invalid layout here.  Such a class
+      ;; cannot have a LAYOUT-INVALID of (:FLUSH ...) or (:OBSOLETE
+      ;; ...), because part of the deal is that those only happen from
+      ;; FORCE-CACHE-FLUSHES, which create a new valid wrapper for the
+      ;; class.  An invalid layout of T needs to be flushed, however.
+      (when (eq (layout-invalid (class-wrapper class)) t)
+        (%force-cache-flushes class))
+      (setf (ctor-class ctor) class)
+      (pushnew ctor (plist-value class 'allocators) :test #'eq)
+      (multiple-value-bind (form optimizedp)
+          (allocator-function-form ctor)
+        (setf (funcallable-instance-fun ctor)
+              (let ((*compiling-optimized-constructor* t))
+                (handler-bind ((compiler-note #'muffle-warning))
+                  (compile nil form)))
+              (ctor-state ctor) (if optimizedp 'optimized 'fallback))))))
+
+(defun allocator-function-form (ctor)
+  (let ((class (ctor-class ctor)))
+    (if (and (not (structure-class-p class))
+             (not (condition-class-p class))
+             (singleton-p (compute-applicable-methods #'allocate-instance
+                                                      (list class)))
+             (every (lambda (x)
+                      (member (slot-definition-allocation x)
+                              '(:instance :class)))
+                    (class-slots class)))
+        (values (optimizing-allocator-generator ctor) t)
+        (values `(lambda ()
+                   (declare #.*optimize-speed*
+                            (notinline allocate-instance))
+                   (allocate-instance ,class))
+                nil))))
+
 (defun constructor-function-form (ctor)
   (let* ((class (ctor-class ctor))
          (proto (class-prototype class))
@@ -594,7 +733,7 @@
                             (safe-method-qualifiers method))
         when (or (and (eq :around (car qualifiers))
                       (not (simple-next-method-call-p method)))
-              (and (null qualifiers)
+                 (and (null qualifiers)
                       (not primary-checked-p)
                       (not (null standard-method))
                       (not (eq standard-method method))))
@@ -649,6 +788,17 @@
        names
        t))))
 
+(defun optimizing-allocator-generator
+    (ctor)
+  (let ((wrapper (class-wrapper (ctor-class ctor))))
+    `(lambda ()
+       (declare #.*optimize-speed*)
+       (block nil
+         (when (layout-invalid ,wrapper)
+           (install-initial-constructor ,ctor)
+           (return (funcall ,ctor)))
+         ,(wrap-in-allocate-forms ctor nil t)))))
+
 ;;; Return a form wrapped around BODY that allocates an instance constructed
 ;;; by CTOR. EARLY-UNBOUND-MARKERS-P means slots may be accessed before we
 ;;; have explicitly initialized them, requiring all slots to start as
@@ -660,8 +810,7 @@
          (allocation-function (raw-instance-allocator class))
          (slots-fetcher (slots-fetcher class)))
     (if (eq allocation-function 'allocate-standard-instance)
-        `(let ((.instance. (%make-standard-instance nil
-                                                    (get-instance-hash-code)))
+        `(let ((.instance. (%make-standard-instance nil 0))
                (.slots. (make-array
                          ,(layout-length wrapper)
                          ,@(when early-unbound-markers-p
