@@ -39,6 +39,10 @@
 ;;; so subsequently compiled code should refer to "SB-PCL", not "SB!PCL".
 (define-symbol-macro *pcl-package* (load-time-value (find-package "SB-PCL") t))
 
+(declaim (inline class-classoid))
+(defun class-classoid (class)
+  (layout-classoid (class-wrapper class)))
+
 (declaim (inline defstruct-classoid-p))
 (defun defstruct-classoid-p (classoid)
   ;; It is non-obvious to me why STRUCTURE-CLASSOID-P doesn't
@@ -69,11 +73,6 @@
   (format-symbol (load-time-value (find-package "SB!PCL") t)
                  "*THE-CLASS-~A*" (symbol-name class-name)))
 
-(defun make-wrapper-symbol (class-name)
-  ;; Reference a package that is now SB!PCL but later SB-PCL
-  (format-symbol (load-time-value (find-package "SB!PCL") t)
-                 "*THE-WRAPPER-~A*" (symbol-name class-name)))
-
 (defun condition-type-p (type)
   (and (symbolp type)
        (condition-classoid-p (find-classoid type nil))))
@@ -92,6 +91,7 @@
                   *the-class-slot-object*
                   *the-class-structure-object*
                   *the-class-standard-object*
+                  *the-class-function*
                   *the-class-funcallable-standard-object*
                   *the-class-class*
                   *the-class-generic-function*
@@ -119,15 +119,37 @@
 
                   *the-eslotd-standard-class-slots*
                   *the-eslotd-funcallable-standard-class-slots*))
+;;;; PCL instances
 
-(declaim (special *the-wrapper-of-t*
-                  *the-wrapper-of-vector* *the-wrapper-of-symbol*
-                  *the-wrapper-of-string* *the-wrapper-of-sequence*
-                  *the-wrapper-of-rational* *the-wrapper-of-ratio*
-                  *the-wrapper-of-number* *the-wrapper-of-null*
-                  *the-wrapper-of-list* *the-wrapper-of-integer*
-                  *the-wrapper-of-float* *the-wrapper-of-cons*
-                  *the-wrapper-of-complex* *the-wrapper-of-character*
-                  *the-wrapper-of-bit-vector* *the-wrapper-of-array*))
+(sb!kernel::!defstruct-with-alternate-metaclass standard-instance
+  :slot-names (slots hash-code)
+  :boa-constructor %make-standard-instance
+  :superclass-name t
+  :metaclass-name standard-classoid
+  :metaclass-constructor make-standard-classoid
+  :dd-type structure
+  :runtime-type-checks-p nil)
+
+(sb!kernel::!defstruct-with-alternate-metaclass standard-funcallable-instance
+  ;; KLUDGE: Note that neither of these slots is ever accessed by its
+  ;; accessor name as of sbcl-0.pre7.63. Presumably everything works
+  ;; by puns based on absolute locations. Fun fun fun.. -- WHN 2001-10-30
+  :slot-names (clos-slots hash-code)
+  :boa-constructor %make-standard-funcallable-instance
+  :superclass-name function
+  :metaclass-name standard-classoid
+  :metaclass-constructor make-standard-classoid
+  :dd-type funcallable-structure
+  ;; Only internal implementation code will access these, and these
+  ;; accesses (slot readers in particular) could easily be a
+  ;; bottleneck, so it seems reasonable to suppress runtime type
+  ;; checks.
+  ;;
+  ;; (Except note KLUDGE above that these accessors aren't used at all
+  ;; (!) as of sbcl-0.pre7.63, so for now it's academic.)
+  :runtime-type-checks-p nil)
+
+(defconstant std-instance-hash-slot-index 2)
+(defconstant fsc-instance-hash-slot-index 2)
 
 (/show0 "finished with early-low.lisp")
