@@ -146,6 +146,7 @@
 ;;; all.
 (macrolet ((def (class)
              `(defmethod class-prototype ((class ,class))
+                (declare (notinline allocate-instance))
                 (with-slots (prototype) class
                   (or prototype
                       (setf prototype (allocate-instance class)))))))
@@ -560,17 +561,23 @@
 (defmethod reinitialize-instance :after ((class condition-class) &key)
   (let* ((name (class-name class))
          (classoid (find-classoid name))
-         (slots (condition-classoid-slots classoid)))
+         (slots (condition-classoid-slots classoid))
+         (source (sb-kernel::layout-source-location (classoid-layout classoid))))
     ;; to balance the REMOVE-SLOT-ACCESSORS call in
     ;; REINITIALIZE-INSTANCE :BEFORE (SLOT-CLASS).
-    (dolist (slot slots)
-      (let ((slot-name (condition-slot-name slot)))
-        (dolist (reader (condition-slot-readers slot))
-          ;; FIXME: see comment in SHARED-INITIALIZE :AFTER
-          ;; (CONDITION-CLASS T), below.  -- CSR, 2005-11-18
-          (sb-kernel::install-condition-slot-reader reader name slot-name))
-        (dolist (writer (condition-slot-writers slot))
-          (sb-kernel::install-condition-slot-writer writer name slot-name))))))
+    (flet ((add-source-location (method)
+             (when source
+               (setf (slot-value method 'source) source))))
+     (dolist (slot slots)
+       (let ((slot-name (condition-slot-name slot)))
+         (dolist (reader (condition-slot-readers slot))
+           ;; FIXME: see comment in SHARED-INITIALIZE :AFTER
+           ;; (CONDITION-CLASS T), below.  -- CSR, 2005-11-18
+           (add-source-location
+            (sb-kernel::install-condition-slot-reader reader name slot-name)))
+         (dolist (writer (condition-slot-writers slot))
+           (add-source-location
+            (sb-kernel::install-condition-slot-writer writer name slot-name))))))))
 
 (defmethod shared-initialize :after ((class condition-class) slot-names
                                      &key direct-slots direct-superclasses)
@@ -728,6 +735,7 @@
 ;;; The way to fix that is to ensure that every defstruct has a zero-argument
 ;;; constructor made by the compiler and stashed in a random symbol.
 (defun make-defstruct-allocation-function (name class)
+  (declare (muffle-conditions code-deletion-note))
   ;; FIXME: Why don't we go class->layout->info == dd
   (let ((dd (find-defstruct-description name)))
     (ecase (dd-type dd)
@@ -1605,6 +1613,7 @@
 
 
 (defun %change-class (instance new-class initargs)
+  (declare (notinline allocate-instance))
   (binding* ((old-class (class-of instance))
              (copy (allocate-instance new-class))
              (new-wrapper (get-wrapper copy))
@@ -1612,12 +1621,13 @@
              (old-slots (get-slots instance))
              (new-slots (get-slots copy))
              (safe (safe-p new-class))
-             (new-instance-slots
-              (classify-slotds (layout-slot-list new-wrapper)))
-             ((old-instance-slots old-class-slots)
-              (classify-slotds (layout-slot-list old-wrapper))))
-    (labels ((find-slot (name slots)
-               (find name slots :key #'slot-definition-name))
+             (new-wrapper-slots (layout-slot-list new-wrapper))
+             (old-wrapper-slots (layout-slot-list old-wrapper)))
+    (labels ((find-instance-slot (name slots)
+               (loop for slot in slots
+                     when (and (eq (slot-definition-allocation slot) :instance)
+                               (eq (slot-definition-name slot) name))
+                     return slot))
              (initarg-for-slot-p (slot)
                (dolist (slot-initarg (slot-definition-initargs slot))
                  ;; Abuse +slot-unbound+
@@ -1632,20 +1642,22 @@
       ;; "The values of local slots specified by both the class CTO
       ;; and CFROM are retained. If such a local slot was unbound, it
       ;; remains unbound."
-      (dolist (new new-instance-slots)
-        (unless (initarg-for-slot-p new)
-          (binding* ((old (find-slot (slot-definition-name new) old-instance-slots)
+      (dolist (new new-wrapper-slots)
+        (when (and (not (initarg-for-slot-p new))
+                   (eq (slot-definition-allocation new) :instance))
+          (binding* ((old (find-instance-slot (slot-definition-name new) old-wrapper-slots)
                           :exit-if-null)
                      (value (clos-slots-ref old-slots (slot-definition-location old))))
             (set-value value new))))
 
       ;; "The values of slots specified as shared in the class CFROM and
       ;; as local in the class CTO are retained."
-      (dolist (old old-class-slots)
-        (binding* ((slot-and-val (slot-definition-location old))
-                   (new (find-slot (car slot-and-val) new-instance-slots)
-                        :exit-if-null))
-          (set-value (cdr slot-and-val) new))))
+      (dolist (old old-wrapper-slots)
+        (when (eq (slot-definition-allocation old) :class)
+         (binding* ((slot-and-val (slot-definition-location old))
+                    (new (find-instance-slot (car slot-and-val) new-wrapper-slots)
+                         :exit-if-null))
+           (set-value (cdr slot-and-val) new)))))
 
     ;; Make the copy point to the old instance's storage, and make the
     ;; old instance point to the new storage.
