@@ -66,6 +66,7 @@
 ;;; don't necessarily always make a wrapper. Also, we help maintain
 ;;; the mapping between CL:CLASS and SB-KERNEL:CLASSOID objects.
 (defun make-wrapper (length class)
+  (declare (notinline slot-value))
   (cond
     ((or (typep class 'std-class)
          (typep class 'forward-referenced-class))
@@ -225,9 +226,7 @@
                         (t
                          (bug "unrecognized instance type")))))
                (:obsolete
-                (%obsolete-instance-trap owrapper (cadr state) instance))))
-            (t
-             (bug "Invalid LAYOUT-INVALID: ~S" state))))))
+                (%obsolete-instance-trap owrapper (cadr state) instance))))))))
 
 (declaim (inline check-obsolete-instance))
 (defun check-obsolete-instance (instance)
