@@ -27,8 +27,10 @@
 #include "lispregs.h"
 #include "genesis/static-symbols.h"
 #include "genesis/vector.h"
+#include "genesis/code.h"
 #include "thread.h"
 #include "monitor.h"
+#include "breakpoint.h"
 
 /* the way that we shut down the system on a fatal error */
 
@@ -39,36 +41,136 @@ default_lossage_handler(void)
 }
 static void (*lossage_handler)(void) = default_lossage_handler;
 
+#if QSHOW
+static void
+configurable_lossage_handler()
+{
+    void lisp_backtrace(int frames);
+
+    if (dyndebug_config.dyndebug_backtrace_when_lost) {
+        fprintf(stderr, "lose: backtrace follows as requested\n");
+        lisp_backtrace(100);
+    }
+
+    if (dyndebug_config.dyndebug_sleep_when_lost) {
+        fprintf(stderr,
+"The system is too badly corrupted or confused to continue at the Lisp.\n"
+"level.  The monitor was enabled, but you requested `sleep_when_lost'\n"
+"behaviour though dyndebug.  To help with your debugging effort, this\n"
+"thread will not enter the monitor, and instead proceed immediately to an\n"
+"infinite sleep call, maximizing your chances that the thread's current\n"
+"state can be preserved until you attach an external debugger. Good luck!\n");
+        for (;;)
+#         ifdef LISP_FEATURE_WIN32
+            Sleep(10000);
+#         else
+            sleep(10);
+#         endif
+    }
+
+    monitor_or_something();
+}
+#endif
+
 void enable_lossage_handler(void)
 {
+#if QSHOW
+    lossage_handler = configurable_lossage_handler;
+#else
     lossage_handler = monitor_or_something;
+#endif
 }
 void disable_lossage_handler(void)
 {
     lossage_handler = default_lossage_handler;
 }
 
-void
-lose(char *fmt, ...)
+static
+void print_message(char *fmt, va_list ap)
 {
-    va_list ap;
-    fprintf(stderr, "fatal error encountered in SBCL pid %d",getpid());
+    fprintf(stderr, " in SBCL pid %d",getpid());
 #if defined(LISP_FEATURE_SB_THREAD)
-    fprintf(stderr, "(tid %lu)", (unsigned long) thread_self());
+    fprintf(stderr, "(tid %lu)", (uword_t) thread_self());
 #endif
     if (fmt) {
         fprintf(stderr, ":\n");
-        va_start(ap, fmt);
         vfprintf(stderr, fmt, ap);
-        va_end(ap);
     }
     fprintf(stderr, "\n");
-    fflush(stderr);
+}
+
+static inline void
+call_lossage_handler() never_returns;
+
+static inline void
+call_lossage_handler()
+{
     lossage_handler();
     fprintf(stderr, "Argh! lossage_handler() returned, total confusion..\n");
     exit(1);
 }
+
+void
+lose(char *fmt, ...)
+{
+    va_list ap;
+    /* Block signals to prevent other threads, timers and such from
+     * interfering. If only all threads could be stopped somehow. */
+    block_blockable_signals(0);
+    fprintf(stderr, "fatal error encountered");
+    va_start(ap, fmt);
+    print_message(fmt, ap);
+    va_end(ap);
+    fprintf(stderr, "\n");
+    fflush(stderr);
+    call_lossage_handler();
+}
+
+boolean lose_on_corruption_p = 0;
+
+void
+corruption_warning_and_maybe_lose(char *fmt, ...)
+{
+    va_list ap;
+#ifndef LISP_FEATURE_WIN32
+    sigset_t oldset;
+    block_blockable_signals(&oldset);
+#endif
+    fprintf(stderr, "CORRUPTION WARNING");
+    va_start(ap, fmt);
+    print_message(fmt, ap);
+    va_end(ap);
+    fprintf(stderr, "The integrity of this image is possibly compromised.\n");
+    if (lose_on_corruption_p)
+        fprintf(stderr, "Exiting.\n");
+    else
+        fprintf(stderr, "Continuing with fingers crossed.\n");
+    fflush(stderr);
+    if (lose_on_corruption_p)
+        call_lossage_handler();
+#ifndef LISP_FEATURE_WIN32
+    else
+        thread_sigmask(SIG_SETMASK,&oldset,0);
+#endif
+}
 
+void print_constant(os_context_t *context, int offset) {
+    lispobj code = find_code(context);
+    if (code != NIL) {
+        struct code *codeptr = (struct code *)native_pointer(code);
+        int length = HeaderValue(codeptr->header);
+        putchar('\t');
+        if (offset >= length) {
+            printf("Constant offset %d out of bounds for the code object of length %d.\n",
+                   offset, length);
+        } else {
+            brief_print(codeptr->constants[offset -
+                                           (offsetof(struct code, constants) >> WORD_SHIFT)]);
+        }
+    }
+}
+
+char *internal_error_descriptions[] = {INTERNAL_ERROR_NAMES};
 /* internal error handler for when the Lisp error system doesn't exist
  *
  * FIXME: Shouldn't error output go to stderr instead of stdout? (Alas,
@@ -80,9 +182,23 @@ describe_internal_error(os_context_t *context)
     unsigned char *ptr = arch_internal_error_arguments(context);
     int len, scoffset, sc, offset, ch;
 
+#ifdef LISP_FEATURE_ARM64
+    u32 trap_instruction = *(u32 *)ptr;
+    unsigned code = trap_instruction >> 13 & 0xFF;
+    printf("Internal error #%d \"%s\" at %p\n", code,
+           internal_error_descriptions[code],
+           (void*)*os_context_pc_addr(context));
+    ptr += 4;
     len = *ptr++;
-    printf("internal error #%d\n", *ptr++);
+#else
+    len = *ptr++;
+    printf("Internal error #%d \"%s\" at %p\n", *ptr,
+           internal_error_descriptions[*ptr],
+           (void*)*os_context_pc_addr(context));
+    ptr++;
     len--;
+#endif
+
     while (len > 0) {
         scoffset = *ptr++;
         len--;
@@ -100,8 +216,8 @@ describe_internal_error(os_context_t *context)
             ptr += 4;
             len -= 4;
         }
-        sc = scoffset & 0x1f;
-        offset = scoffset >> 5;
+        sc = scoffset & 0x3F;
+        offset = scoffset >> 6;
 
         printf("    SC: %d, Offset: %d", sc, offset);
         switch (sc) {
@@ -153,6 +269,9 @@ describe_internal_error(os_context_t *context)
             printf("\t%g\n", *(double *)&context->sc_fpregs[offset]);
             break;
 #endif
+        case sc_Constant:
+            print_constant(context, offset);
+            break;
         default:
             printf("\t???\n");
             break;
