@@ -435,31 +435,21 @@ check_gc_signals_blocked_or_lose(sigset_t *sigset)
 #endif
 
 void
-block_deferrable_signals(sigset_t *where, sigset_t *old)
+block_deferrable_signals(sigset_t *old)
 {
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
-    block_signals(&deferrable_sigset, where, old);
+    block_signals(&deferrable_sigset, 0, old);
 #endif
 }
 
 void
-block_blockable_signals(sigset_t *where, sigset_t *old)
+block_blockable_signals(sigset_t *old)
 {
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
-    block_signals(&blockable_sigset, where, old);
+    block_signals(&blockable_sigset, 0, old);
 #endif
 }
 
-#ifndef LISP_FEATURE_SB_SAFEPOINT
-void
-block_gc_signals(sigset_t *where, sigset_t *old)
-{
-#if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
-    block_signals(&gc_sigset, where, old);
-#endif
-}
-#endif
-
 void
 unblock_deferrable_signals(sigset_t *where, sigset_t *old)
 {
@@ -473,14 +463,6 @@ unblock_deferrable_signals(sigset_t *where, sigset_t *old)
 #endif
 }
 
-void
-unblock_blockable_signals(sigset_t *where, sigset_t *old)
-{
-#if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
-    unblock_signals(&blockable_sigset, where, old);
-#endif
-}
-
 #ifndef LISP_FEATURE_SB_SAFEPOINT
 void
 unblock_gc_signals(sigset_t *where, sigset_t *old)
@@ -537,7 +519,7 @@ maybe_save_gc_mask_and_block_deferrables(sigset_t *sigset)
     sigset_t oldset;
     /* Obviously, this function is called when signals may not be
      * blocked. Let's make sure we are not interrupted. */
-    block_blockable_signals(0, &oldset);
+    block_blockable_signals(&oldset);
 #ifndef LISP_FEATURE_SB_THREAD
     /* With threads a SIG_STOP_FOR_GC and a normal GC may also want to
      * block. */
@@ -796,7 +778,7 @@ undo_fake_foreign_function_call(os_context_t *context)
 {
     struct thread *thread=arch_os_get_current_thread();
     /* Block all blockable signals. */
-    block_blockable_signals(0, 0);
+    block_blockable_signals(0);
 
     foreign_function_call_active_p(thread) = 0;
 
@@ -1482,7 +1464,9 @@ arrange_return_to_c_function(os_context_t *context,
      * must obviously exist in reality.  That would be post_signal_tramp
      */
 
+#ifndef LISP_FEATURE_DARWIN
     u32 *sp=(u32 *)*os_context_register_addr(context,reg_ESP);
+#endif
 
 #if defined(LISP_FEATURE_DARWIN)
     u32 *register_save_area = (u32 *)os_validate(0, 0x40);
@@ -1619,7 +1603,9 @@ void
 arrange_return_to_lisp_function(os_context_t *context, lispobj function)
 {
 #if defined(LISP_FEATURE_DARWIN) && defined(LISP_FEATURE_X86)
-    arrange_return_to_c_function(context, call_into_lisp_tramp, function);
+    arrange_return_to_c_function(context,
+                                 (call_into_lisp_lookalike)call_into_lisp_tramp,
+                                 function);
 #else
     arrange_return_to_c_function(context, call_into_lisp, function);
 #endif
@@ -2013,7 +1999,7 @@ install_handler(int signal, void handler(int, siginfo_t*, os_context_t*),
 
     FSHOW((stderr, "/entering POSIX install_handler(%d, ..)\n", signal));
 
-    block_blockable_signals(0, &old);
+    block_blockable_signals(&old);
 
     FSHOW((stderr, "/interrupt_low_level_handlers[signal]=%p\n",
            interrupt_low_level_handlers[signal]));
