@@ -93,11 +93,19 @@
 
 ;;; Following refactoring of sequence functions to detect bad type
 ;;; specifiers, REVERSE was left broken on vectors with fill pointers.
-(let ((a (make-array 10
-                     :fill-pointer 5
-                     :element-type 'character
-                     :initial-contents "abcdefghij")))
-  (assert (string= (reverse a) "edcba")))
+(with-test (:name :reverse-fill-pointer.string)
+  (let ((a (make-array 10
+                       :fill-pointer 5
+                       :element-type 'character
+                       :initial-contents "abcdefghij")))
+    (assert (string= (reverse a) "edcba"))))
+
+(with-test (:name :reverse-fill-pointer.fixnum)
+  (let ((a (make-array 10
+                       :fill-pointer 6
+                       :element-type 'fixnum
+                       :initial-contents '(0 1 2 3 4 5 7 8 9 10))))
+    (assert (equalp (reverse a) #(5 4 3 2 1 0)))))
 
 ;;; ARRAY-IN-BOUNDS-P should work when given non-INDEXes as its
 ;;; subscripts (and return NIL, of course)
@@ -397,3 +405,44 @@
   (let ((array (make-array 10 :fill-pointer t)))
     (assert (= (fill-pointer (adjust-array array 5 :fill-pointer 2))
                2))))
+
+(with-test (:name :adjust-array-initial-element)
+  (assert (equal (funcall
+                  (checked-compile
+                   `(lambda (x)
+                      (adjust-array x 5 :initial-element #\x)))
+                  "abc")
+                 "abcxx")))
+
+(with-test (:name :array-initial-contents-1)
+  (flet ((f (x y)
+           (sb-int:dx-let ((a (make-array `(,x ,y)
+                                          :initial-contents
+                                          '((a b c) (1 2 3)))))
+             (eval a)
+             nil)))
+    (f 2 3)
+    (assert-error (f 3 2))))
+
+(with-test (:name :array-initial-contents-2)
+  (labels ((compute-contents () '((a b c) (1 2 3)))
+           (f (x y)
+             (sb-int:dx-let ((a (make-array `(,x ,y)
+                                            :initial-contents
+                                            (compute-contents))))
+               (eval a)
+                nil)))
+    (declare (notinline compute-contents))
+    (f 2 3)
+    (assert-error (f 3 2))))
+
+(with-test (:name :array-initial-contents-3)
+  (multiple-value-bind (f warningp errorp)
+      ;; FIXME: should be CHECKED-COMPILE
+      (let ((*error-output* (make-broadcast-stream)))
+        (compile nil '(lambda (z)
+                        (symbol-macrolet ((x (+ 1 1)) (y (* 2 1)))
+                          (make-array `(,x ,y)
+                                      :initial-contents
+                                      `((,z ,z 1) (,z ,z ,z)))))))
+    (assert (and f warningp errorp))))
