@@ -299,8 +299,8 @@
 
     ;; ftype
     ;;
-    ;; The interpreter doesn't do anything with ftype declarations
-    #+#.(cl:if (cl:eq sb-ext:*evaluator-mode* :compile) '(and) '(or))
+    ;; The legacy interpreter doesn't do anything with ftype declarations
+    #+(or :sb-fasteval (not :interpreter))
     (test:function . (locally
                          (declare (ftype function test:function))
                        (cons t t)))
@@ -308,7 +308,7 @@
     ;; type
     ;;
     ;; Nor with type declarations
-    #+#.(cl:if (cl:eq sb-ext:*evaluator-mode* :compile) '(and) '(or))
+    #+(or :sb-fasteval (not :interpreter))
     (test:num . (locally
                     (declare (type fixnum test:num))
                   (cons t t)))
@@ -319,7 +319,7 @@
                         (cons t t)))
 
     ;; declare ftype
-    #+#.(cl:if (cl:eq sb-ext:*evaluator-mode* :compile) '(and) '(or))
+    #+(or :sb-fasteval (not :interpreter))
     (test:numfun . (locally
                        (declare (ftype (function (fixnum) fixnum) test:numfun))
                      (cons t t)))))
@@ -362,7 +362,12 @@
       (with-error-info ("compile locked illegal lexical form: ~S~%" form)
         (let ((fun (compile nil `(lambda () ,form))))
           (assert-error (funcall fun) program-error))
-        (assert-error (eval form) program-error)))))
+        (assert-error (eval form)
+                      ;; Let's not be pedantic here.
+                      ;; PACKAGE-LOCK-VIOLATION is right,
+                      ;; because the distinction between lexical analysis
+                      ;; and running is artificial for interpreted code.
+                      (or sb-ext:package-lock-violation program-error))))))
 
 ;;; Locked, WITHOUT-PACKAGE-LOCKS
 (reset-test t)
@@ -456,7 +461,7 @@
               ,form
               (locally (declare (enable-package-locks ,sym))
                 ,form)))
-     program-error)))
+     (or sb-ext:package-lock-violation program-error))))
 
 ;;;; See that trace on functions in locked packages doesn't break
 ;;;; anything.
@@ -487,7 +492,8 @@
   test:*special*)
 (assert (eq *terminal-io* (pcl-type-declaration-method-bug *terminal-io*)))
 
-#+#.(cl:if (cl:eq sb-ext:*evaluator-mode* :compile) '(and) '(or))
+;; Interpreters don't walk into a method body until it's executed.
+#-:interpreter
 (assert-error
  (eval
   '(defmethod pcl-type-declaration-method-bug ((test:*special* stream))
